# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from rest_framework import serializers

from koalixcrm.accounting.accounting.accounting_period import AccountingPeriod
from koalixcrm.accounting.accounting.booking import Booking
from koalixcrm.accounting.models import Account
from koalixcrm.accounting.rest.account_rest import OptionAccountJSONSerializer
from koalixcrm.accounting.rest.accounting_period_rest import OptionAccountingPeriodJSONSerializer


class UserJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    username = serializers.CharField(read_only=True)
    firstName = serializers.CharField(source='first_name', read_only=True)
    lastName = serializers.CharField(source='last_name', read_only=True)

    class Meta:
        model = User
        fields = (
            'id',
            'username',
            'firstName',
            'lastName')


class BookingJSONSerializer(serializers.HyperlinkedModelSerializer):

    fromAccount = OptionAccountJSONSerializer(source='from_account', allow_null=False)
    toAccount = OptionAccountJSONSerializer(source='to_account', allow_null=False)
    bookingDate = serializers.DateTimeField(source='booking_date', format="%Y-%m-%dT%H:%M", input_formats=None,
                                            allow_null=False)
    bookingReference = serializers.CharField(source='booking_reference', allow_null=True)
    accountingPeriod = OptionAccountingPeriodJSONSerializer(source='accounting_period', allow_null=False)
    staff = UserJSONSerializer(required=False, allow_null=True)

    class Meta:
        model = Booking
        fields = ('id',
                  'fromAccount',
                  'toAccount',
                  'description',
                  'amount',
                  'bookingDate',
                  'staff',
                  'bookingReference',
                  'accountingPeriod')
        depth = 1

    def create(self, validated_data):
        booking = Booking()
        booking.description = validated_data['description']
        booking.amount = validated_data['amount']
        booking.booking_date = validated_data['booking_date']
        booking.booking_reference = validated_data['booking_reference']

        # Deserialize from staff
        request = self.context.get('request')
        koalixcrm_user = request.META.get('HTTP_KOALIXCRM_USER')
        user = User.objects.get(username=koalixcrm_user)
        booking.staff = user
        booking.last_modified_by = user

        # Deserialize from account
        from_account = validated_data.pop('from_account')
        if from_account:
            if from_account.get('id', None):
                booking.from_account = Account.objects.get(id=from_account.get('id', None))
            else:
                booking.from_account = None

        # Deserialize to account
        to_account = validated_data.pop('to_account')
        if to_account:
            if to_account.get('id', None):
                booking.to_account = Account.objects.get(id=to_account.get('id', None))
            else:
                booking.to_account = None

        # Deserialize to accounting period
        accounting_period = validated_data.pop('accounting_period')
        if accounting_period:
            if accounting_period.get('id', None):
                booking.accounting_period = AccountingPeriod.objects.get(id=accounting_period.get('id', None))
            else:
                booking.accounting_period = None

        booking.save()
        return booking

    def update(self, booking, validated_data):
        booking.description = validated_data['description']
        booking.amount = validated_data['amount']
        booking.booking_date = validated_data['booking_date']
        booking.booking_reference = validated_data['booking_reference']

        # Deserialize from staff
        request = self.context.get('request')
        koalixcrm_user = request.META.get('HTTP_KOALIXCRM_USER')
        user = User.objects.get(username=koalixcrm_user)
        booking.staff = user

        # Deserialize from account
        from_account = validated_data.pop('from_account')
        if from_account:
            if from_account.get('id', booking.from_account):
                booking.from_account = Account.objects.get(id=from_account.get('id', None))
            else:
                booking.from_account = booking.from_account_id
        else:
            booking.from_account = None

        # Deserialize to account
        to_account = validated_data.pop('to_account')
        if to_account:
            if to_account.get('id', booking.to_account):
                booking.to_account = Account.objects.get(id=to_account.get('id', None))
            else:
                booking.to_account = booking.to_account_id
        else:
            booking.to_account = None

        # Deserialize to accounting period
        accounting_period = validated_data.pop('accounting_period')
        if accounting_period:
            if accounting_period.get('id', booking.accounting_period):
                booking.accounting_period = AccountingPeriod.objects.get(id=accounting_period.get('id', None))
            else:
                booking.accounting_period = booking.accounting_period_id
        else:
            booking.accounting_period = None

        booking.save()
        return booking



# -*- coding: utf-8 -*-
from rest_framework import serializers

from koalixcrm.crm.contact.contact import Contact, PhoneAddressForContact, PostalAddressForContact, \
    EmailAddressForContact
from koalixcrm.crm.contact.email_address import EmailAddress
from koalixcrm.crm.contact.phone_address import PhoneAddress
from koalixcrm.crm.contact.postal_address import PostalAddress


class PhoneAddressJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = PhoneAddress
        fields = ('phone',)


class ContactPhoneAddressJSONSerializer(PhoneAddressJSONSerializer):
    contactId = serializers.PrimaryKeyRelatedField(source='person_id', queryset=Contact.objects.all())

    class Meta:
        model = PhoneAddressForContact
        fields = PhoneAddressJSONSerializer.Meta.fields + ('id', 'purpose', 'contactId')

    def create(self, validated_data):
        contact_phone_address = PhoneAddressForContact()

        contact = validated_data.pop('person_id')
        contact_phone_address.person_id = contact.id
        contact_phone_address.purpose = validated_data['purpose']
        contact_phone_address.phone = validated_data['phone']

        contact_phone_address.save()
        return contact_phone_address

    def update(self, contact_phone_address, validated_data):
        contact = validated_data.pop('person_id')
        contact_phone_address.person_id = contact.id
        contact_phone_address.purpose = validated_data['purpose']
        contact_phone_address.prefix = validated_data['phone']

        contact_phone_address.save()

        return contact_phone_address


class EmailAddressJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = EmailAddress
        fields = ('email',)


class ContactEmailAddressJSONSerializer(EmailAddressJSONSerializer):
    contactId = serializers.PrimaryKeyRelatedField(source='person_id', queryset=Contact.objects.all())

    class Meta:
        model = EmailAddressForContact
        fields = EmailAddressJSONSerializer.Meta.fields + ('id', 'purpose', 'contactId')

    def create(self, validated_data):
        contact_email_address = EmailAddressForContact()

        contact = validated_data.pop('person_id')
        contact_email_address.person_id = contact.id
        contact_email_address.purpose = validated_data['purpose']
        contact_email_address.email = validated_data['email']

        contact_email_address.save()
        return contact_email_address

    def update(self, contact_email_address, validated_data):
        contact = validated_data.pop('person_id')
        contact_email_address.person_id = contact.id
        contact_email_address.purpose = validated_data['purpose']
        contact_email_address.email = validated_data['email']

        contact_email_address.save()

        return contact_email_address


class ContactJSONSerializer(serializers.HyperlinkedModelSerializer):
    state = serializers.SerializerMethodField()
    city = serializers.SerializerMethodField('get_town')

    class Meta:
        model = Contact
        fields = ('name',
                  'state',
                  'city')

    @staticmethod
    def get_postal_address(obj):
        return PostalAddressForContact.objects.filter(person=obj.id).first()

    def get_state(self, obj):
        address = self.get_postal_address(obj)
        return address.state if address is not None else None

    def get_town(self, obj):
        address = self.get_postal_address(obj)
        return address.town if address is not None else None


class PostalAddressJSONSerializer(serializers.HyperlinkedModelSerializer):
    givenName = serializers.CharField(source='pre_name', allow_null=True)
    familyName = serializers.CharField(source='name', allow_null=True)
    addressLine1 = serializers.CharField(source='address_line_1', allow_null=True)
    addressLine2 = serializers.CharField(source='address_line_2', allow_null=True)
    addressLine3 = serializers.CharField(source='address_line_3', allow_null=True)
    addressLine4 = serializers.CharField(source='address_line_4', allow_null=True)
    zipCode = serializers.IntegerField(source='zip_code',  allow_null=True)
    city = serializers.CharField(source='town',  allow_null=True)

    class Meta:
        model = PostalAddress
        fields = ('prefix',
                  'familyName',
                  'givenName',
                  'addressLine1',
                  'addressLine2',
                  'addressLine3',
                  'addressLine4',
                  'zipCode',
                  'city',
                  'state',
                  'country')


class ContactPostalAddressJSONSerializer(PostalAddressJSONSerializer):
    contactId = serializers.PrimaryKeyRelatedField(source='person_id', queryset=Contact.objects.all())

    class Meta:
        model = PostalAddressForContact
        fields = PostalAddressJSONSerializer.Meta.fields + ('id', 'purpose', 'contactId')

    def create(self, validated_data):
        contact_postal_address = PostalAddressForContact()

        contact = validated_data.pop('person_id')
        contact_postal_address.person_id = contact.id
        contact_postal_address.purpose = validated_data['purpose']
        contact_postal_address.prefix = validated_data['prefix']
        contact_postal_address.name = validated_data['name']
        contact_postal_address.name = validated_data['pre_name']
        contact_postal_address.address_line_1 = validated_data['address_line_1']
        contact_postal_address.address_line_2 = validated_data['address_line_2']
        contact_postal_address.address_line_3 = validated_data['address_line_3']
        contact_postal_address.address_line_4 = validated_data['address_line_4']
        contact_postal_address.zip_code = validated_data['zip_code']
        contact_postal_address.town = validated_data['town']
        contact_postal_address.state = validated_data['state']
        contact_postal_address.country = validated_data['country']

        contact_postal_address.save()
        return contact_postal_address

    def update(self, contact_postal_address, validated_data):
        contact = validated_data.pop('person_id')
        contact_postal_address.person_id = contact.id
        contact_postal_address.purpose = validated_data['purpose']
        contact_postal_address.prefix = validated_data['prefix']
        contact_postal_address.name = validated_data['name']
        contact_postal_address.pre_name = validated_data['pre_name']
        contact_postal_address.address_line_1 = validated_data['address_line_1']
        contact_postal_address.address_line_2 = validated_data['address_line_2']
        contact_postal_address.address_line_3 = validated_data['address_line_3']
        contact_postal_address.address_line_4 = validated_data['address_line_4']
        contact_postal_address.zip_code = validated_data['zip_code']
        contact_postal_address.town = validated_data['town']
        contact_postal_address.state = validated_data['state']
        contact_postal_address.country = validated_data['country']

        contact_postal_address.save()

        return contact_postal_address




# -*- coding: utf-8 -*-
from django.contrib import admin
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from koalixcrm.subscriptions.models import *


class AdminSubscriptionEvent(admin.TabularInline):
    model = SubscriptionEvent
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        ('Basics', {
            'fields': (
                'event_date',
                'event',)
        }),
    )
    allow_add = True


class InlineSubscription(admin.TabularInline):
    model = Subscription
    extra = 1
    classes = ('collapse-open',)
    readonly_fields = ('contract',
                       'subscription_type')
    fieldsets = (
        (_('Basics'), {
            'fields': (
                'contract',
                'subscription_type')
        }),
    )
    allow_add = False


class OptionSubscription(admin.ModelAdmin):
    list_display = ('id',
                    'contract',
                    'subscription_type',)
    ordering = ('id',
                'contract',
                'subscription_type')
    search_fields = ('id',
                     'contract',)
    fieldsets = (
        (_('Basics'), {
            'fields': (
                'contract',
                'subscription_type',)
        }),
    )
    inlines = [AdminSubscriptionEvent]

    @staticmethod
    def create_invoice(queryset):
        for obj in queryset:
            invoice = obj.create_invoice()
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response
    create_invoice.short_description = _("Create Invoice")

    @staticmethod
    def create_quote(queryset):
        for obj in queryset:
            invoice = obj.create_invoice()
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()

    actions = ['create_subscription_pdf', 'create_invoice']


class OptionSubscriptionType(admin.ModelAdmin):
    list_display = ('id',)
    list_display_links = ('id',)
    ordering = ('id', )
    search_fields = ('id',)
    fieldsets = (
        (_('Basics'), {
            'fields': (
                'cancellation_period',
                'automatic_contract_extension',
                'automatic_contract_extension_reminder',
                'minimum_duration',
                'payment_interval',
                'contract_document')
        }),
    )


def create_subscription(a, request, queryset):
    for contract in queryset:
        subscription = Subscription()
        subscription.create_subscription_from_contract(crmmodels.Contract.objects.get(id=contract.id))
        response = HttpResponseRedirect('/admin/subscriptions/' + str(subscription.id))
    return response


create_subscription.short_description = _("Create Subscription")


class KoalixcrmPluginInterface(object):
    contractInlines = [InlineSubscription]
    contractActions = [create_subscription]
    invoiceInlines = []
    invoiceActions = []
    quoteInlines = []
    quoteActions = []
    customerInlines = []
    customerActions = []


admin.site.register(Subscription, OptionSubscription)
admin.site.register(SubscriptionType, OptionSubscriptionType)



import datetime
import pytz
from django.test import TestCase
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory
from koalixcrm.crm.factories.factory_task_status import DoneTaskStatusFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_estimation import StandardHumanResourceEstimationToTaskFactory


class TaskUpdateLastStatusUpdate(TestCase):

    def setUp(self):
        datetime_now = datetime.datetime(2024, 1, 1, 0, 00)
        datetime_now = pytz.timezone("UTC").localize(datetime_now, is_dst=None)
        start_date = (datetime_now - datetime.timedelta(days=30)).date()
        end_date_first_task = (datetime_now + datetime.timedelta(days=30)).date()
        end_date_second_task = (datetime_now + datetime.timedelta(days=60)).date()

        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_user = AdminUserFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.test_user_extension = StandardUserExtensionFactory.create(user=self.test_user)
        self.test_reporting_period = StandardReportingPeriodFactory.create()
        self.test_1st_task = StandardTaskFactory.create(title="1st Test Task",
                                                        project=self.test_reporting_period.project,
                                                        last_status_change=datetime.date(2024, 6, 15)
                                                        )
        self.estimation_1st_task = StandardHumanResourceEstimationToTaskFactory.create(task=self.test_1st_task,
                                                                                       date_from=start_date,
                                                                                       date_until=end_date_first_task)
        self.test_2nd_task = StandardTaskFactory.create(title="2nd Test Task",
                                                        project=self.test_reporting_period.project,
                                                        last_status_change=datetime.date(2024, 6, 15))
        self.estimation_2nd_task = StandardHumanResourceEstimationToTaskFactory.create(task=self.test_2nd_task,
                                                                                       date_from=start_date,
                                                                                       date_until=end_date_second_task)

    def test_task_last_status_update(self):
        previous_last_status_change = self.test_1st_task.last_status_change
        new_status = DoneTaskStatusFactory.create()
        self.test_1st_task.status = new_status
        self.test_1st_task.save()
        self.assertEquals(previous_last_status_change, datetime.date(2024, 6, 15))
        self.assertEqual(self.test_1st_task.last_status_change, datetime.date.today())



# -*- coding: utf-8 -*-

import datetime
from dateutil.relativedelta import *
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.djangoUserExtension.user_extension.user_extension import UserExtension
from koalixcrm.crm.reporting.resource import Resource
from koalixcrm.crm.reporting.resource_price import ResourcePriceInlineAdminView
from koalixcrm.crm.reporting.work import Work
from koalixcrm.crm.documents.pdf_export import PDFExport


class HumanResource(Resource):
    user = models.ForeignKey(UserExtension,
                             on_delete=models.CASCADE,
                             verbose_name=_("User"))

    def __str__(self):
        return self.user.__str__()

    def serialize_to_xml(self, **kwargs):
        date_from = kwargs.get('date_from', datetime.date.today()-datetime.timedelta(days=60))
        date_to = kwargs.get('date_to', datetime.date.today())
        date_first_of_the_month = date_from.replace(day=1)
        date_first_of_next_month = date_from.replace(day=1) + relativedelta(months=+1)
        date_end_of_the_month = date_first_of_next_month - datetime.timedelta(days=1)
        date = date_first_of_the_month
        days = dict()
        weeks = dict()
        months = dict()
        projects = self.resource_contribution_project(date_from, date_to)
        objects = [self, self.user]
        objects.extend(projects)
        main_xml = PDFExport.write_xml(objects)
        while date < date_from:
            project_efforts = dict()
            for project in projects:
                project_efforts[project] = {'effort': "-",
                                            'project': project.id.__str__()}
            days[date] = {'effort': "-",
                          "day": str(date.day),
                          "week": str(date.isocalendar()[1]),
                          "week_day": str(date.isoweekday()),
                          "month": str(date.month),
                          "year": str(date.year),
                          "project_efforts": project_efforts}
            date += datetime.timedelta(days=1)
        while date <= date_to:
            project_efforts_day = dict()
            project_efforts_week = dict()
            project_efforts_month = dict()
            for project in projects:
                project_efforts_day[project] = {'effort': 0,
                                                'project': project.id.__str__()}
                project_efforts_week[project] = {'effort': 0,
                                                 'project': project.id.__str__()}
                project_efforts_month[project] = {'effort': 0,
                                                  'project': project.id.__str__()}
            days[date] = {'effort': 0,
                          "day": str(date.day),
                          "week": str(date.isocalendar()[1]),
                          "week_day": str(date.isoweekday()),
                          "month": str(date.month),
                          "year": str(date.year),
                          "project_efforts": project_efforts_day}
            month_key = str(date.month)+"/"+str(date.year)
            week_key = str(date.isocalendar()[1])+"/"+str(date.year)
            if not (week_key in weeks):
                weeks[week_key] = {'effort': 0,
                                   'week': str(date.isocalendar()[1]),
                                   'year': str(date.year),
                                   "project_efforts": project_efforts_week}
            if not (month_key in months):
                months[month_key] = {'effort': 0,
                                     'month': str(date.month),
                                     'year': str(date.year),
                                     "project_efforts": project_efforts_month}
            date += datetime.timedelta(days=1)
        while date < date_end_of_the_month:
            project_efforts = dict()
            for project in projects:
                project_efforts[project] = {'effort': "-",
                                            'project': project.id.__str__()}
            days[date] = {'effort': "-",
                          "day": str(date.day),
                          "week": str(date.isocalendar()[1]),
                          "week_day": str(date.isoweekday()),
                          "month": str(date.month),
                          "year": str(date.year),
                          "project_efforts": project_efforts}
            date += datetime.timedelta(days=1)
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       ".",
                                                       "range_from",
                                                       date_from.__str__(),
                                                       attributes={"day": str(date_from.day),
                                                                   "week": str(date_from.isocalendar()[1]),
                                                                   "week_day": str(date_from.isoweekday()),
                                                                   "month": str(date_from.month),
                                                                   "year": str(date_from.year)})
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       ".",
                                                       "range_to",
                                                       date_to.__str__(),
                                                       attributes={"day": str(date_to.day),
                                                                   "week": str(date_to.isocalendar()[1]),
                                                                   "week_day": str(date_to.isoweekday()),
                                                                   "month": str(date_to.month),
                                                                   "year": str(date_to.year)})
        works = Work.objects.filter(human_resource=self, date__range=(date_from, date_to))
        for work in works:
            days[work.date]['effort'] += work.effort_hours()
            days[work.date]['project_efforts'][work.task.project]['effort'] += work.effort_hours()
            month_key = str(work.date.month)+"/"+str(work.date.year)
            week_key = str(work.date.isocalendar()[1])+"/"+str(work.date.year)
            weeks[week_key]['effort'] += work.effort_hours()
            weeks[week_key]['project_efforts'][work.task.project]['effort'] += work.effort_hours()
            months[month_key]['effort'] += work.effort_hours()
            months[month_key]['project_efforts'][work.task.project]['effort'] += work.effort_hours()
            work_xml = work.serialize_to_xml()
            main_xml = PDFExport.merge_xml(main_xml, work_xml)
        for day_key in days.keys():
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='djangoUserExtension.userextension']",
                                                           "Day_Work_Hours",
                                                           str(days[day_key]['effort']),
                                                           attributes={"day": days[day_key]['day'],
                                                                       "week": days[day_key]['week'],
                                                                       "week_day": days[day_key]['week_day'],
                                                                       "month": days[day_key]['month'],
                                                                       "year": days[day_key]['year']})
            for project_key in days[day_key]['project_efforts'].keys():
                main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                               "object/[@model='djangoUserExtension.userextension']",
                                                               "Day_Project_Work_Hours",
                                                               str(days[day_key]['project_efforts'][project_key]['effort']),
                                                               attributes={"day": days[day_key]['day'],
                                                                           "week": days[day_key]['week'],
                                                                           "week_day": days[day_key]['week_day'],
                                                                           "month": days[day_key]['month'],
                                                                           "year": days[day_key]['year'],
                                                                           "project": days[day_key]['project_efforts'][project_key]['project']})
        for week_key in weeks.keys():
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='djangoUserExtension.userextension']",
                                                           "Week_Work_Hours",
                                                           str(weeks[week_key]['effort']),
                                                           attributes={"week": weeks[week_key]['week'],
                                                                       "year": weeks[week_key]['year']})
            for project_key in weeks[week_key]['project_efforts'].keys():
                main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                               "object/[@model='djangoUserExtension.userextension']",
                                                               "Week_Project_Work_Hours",
                                                               str(weeks[week_key]['project_efforts'][project_key]['effort']),
                                                               attributes={"week": weeks[week_key]['week'],
                                                                           "year": weeks[week_key]['year'],
                                                                           "project": weeks[week_key]['project_efforts'][project_key]['project']})
        for month_key in months.keys():
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='djangoUserExtension.userextension']",
                                                           "Month_Work_Hours",
                                                           str(months[month_key]['effort']),
                                                           attributes={"month": months[month_key]['month'],
                                                                       "year": months[month_key]['year']})
            for project_key in months[month_key]['project_efforts'].keys():
                main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                               "object/[@model='djangoUserExtension.userextension']",
                                                               "Month_Project_Work_Hours",
                                                               str(months[month_key]['project_efforts'][project_key]['effort']),
                                                               attributes={"month": months[month_key]['month'],
                                                                           "year": months[month_key]['year'],
                                                                           "project": months[month_key]['project_efforts'][project_key]['project']})
        return main_xml

    def create_pdf(self, template_set, printed_by, *args, **kwargs):
        return PDFExport.create_pdf(self, template_set, printed_by, *args, **kwargs)

    def get_template_set(self, template_set):
        return self.user.get_template_set(template_set)

    def get_fop_config_file(self, template_set):
        return self.user.get_fop_config_file(template_set)

    def get_xsl_file(self, template_set):
        return self.user.get_xsl_file(template_set)

    def resource_contribution_project(self, date_from, date_to):
        works = Work.objects.filter(human_resource=self,
                                    date__range=(date_from, date_to))
        projects = []
        for work in works:
            if work.task.project not in projects:
                projects.append(work.task.project)
        return projects


class HumanResourceAdminView(admin.ModelAdmin):
    list_display = ('id',
                    'user',
                    'resource_manager',
                    'resource_type')
    list_display_links = ('id',
                          'user')
    list_filter = ('user',)
    ordering = ('id',)
    search_fields = ('id',
                     'user')
    fieldsets = (
        (_('Basics'), {
            'fields': ('user',
                       'resource_manager',
                       'resource_type')
        }),
    )

    def create_work_report_pdf(self, request, queryset):
        from koalixcrm.crm.views.create_work_report import create_work_report

        return create_work_report(self, request, queryset)

    create_work_report_pdf.short_description = _("Work Report PDF")

    save_as = True
    actions = [create_work_report_pdf]
    inlines = [ResourcePriceInlineAdminView]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-02 16:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0006_auto_20180701_2128'),
    ]

    operations = [
        migrations.RenameField(
            model_name='userextension',
            old_name='defaultCurrency',
            new_name='default_currency',
        ),
        migrations.RenameField(
            model_name='userextension',
            old_name='defaultTemplateSet',
            new_name='default_template_set',
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _


class Resource(models.Model):
    resource_manager = models.ForeignKey("ResourceManager",
                                         on_delete=models.CASCADE,
                                         verbose_name=_("Manager"),
                                         blank=True,
                                         null=True)
    resource_type = models.ForeignKey("ResourceType",
                                      on_delete=models.CASCADE,
                                      verbose_name=_("Resource Type"),
                                      blank=True,
                                      null=True)

    def __str__(self):
        from koalixcrm.crm.reporting.human_resource import HumanResource
        human_resource = HumanResource.objects.get(id=self.id)
        if human_resource:
            return human_resource.__str__()
        else:
            return "Resource"




# -*- coding: utf-8 -*-

from datetime import *
from django.db import models
from django.utils.translation import ugettext as _
from koalixcrm.crm.const.status import *
from django.core.validators import MaxValueValidator, MinValueValidator
from koalixcrm.crm.documents.sales_document import SalesDocument, OptionSalesDocument
from koalixcrm.plugin import *


class PaymentReminder(SalesDocument):
    payable_until = models.DateField(verbose_name=_("To pay until"))
    payment_bank_reference = models.CharField(verbose_name=_("Payment Bank Reference"),
                                              max_length=100,
                                              blank=True,
                                              null=True)
    iteration_number = models.IntegerField(blank=False,
                                           null=False,
                                           verbose_name=_("Iteration Number"),
                                           validators=[MinValueValidator(1),
                                                       MaxValueValidator(3)])
    status = models.CharField(max_length=1,
                              choices=INVOICESTATUS)

    def create_from_reference(self, calling_model):
        self.create_sales_document(calling_model)
        self.status = 'C'
        self.iteration_number = 1
        self.payable_until = date.today() + \
                             timedelta(days=self.customer.default_customer_billing_cycle.payment_reminder_time_to_payment)
        self.template_set = self.contract.get_template_set(self)
        self.save()
        self.attach_sales_document_positions(calling_model)
        self.attach_text_paragraphs()
        self.staff = calling_model.staff

    def __str__(self):
        return _("Payment Reminder") + ": " + self.id.__str__() + \
               " " + _("from Contract") + ": " + self.contract.id.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Payment Reminder')
        verbose_name_plural = _('Payment Reminders')


class OptionPaymentReminder(OptionSalesDocument):
    list_display = OptionSalesDocument.list_display + ('payable_until',
                                                       'status',
                                                       'iteration_number')
    list_filter = OptionSalesDocument.list_filter + ('status',)
    ordering = OptionSalesDocument.ordering
    search_fields = OptionSalesDocument.search_fields
    fieldsets = OptionSalesDocument.fieldsets + (
        (_('Quote specific'), {
            'fields': ('payable_until',
                       'status',
                       'payment_bank_reference',
                       'iteration_number')
        }),
    )

    save_as = OptionSalesDocument.save_as
    inlines = OptionSalesDocument.inlines
    actions = ['create_purchase_confirmation',
               'create_invoice',
               'create_quote',
               'create_delivery_note',
               'create_pdf',
               'register_invoice_in_accounting',
               'register_payment_in_accounting']

    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("quoteInlines"))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-27 20:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='salescontract',
            name='lastCalculatedPrice',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Last Calculated Price With Tax'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-11 20:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0016_auto_20180111_2011'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailaddressforsalesdocument',
            options={'verbose_name': 'Email Address For Contract', 'verbose_name_plural': 'Email Address For Contracts'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforsalesdocument',
            options={'verbose_name': 'Phone Address For Sales Contract', 'verbose_name_plural': 'Phone Address For Contracts'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforsalesdocument',
            options={'verbose_name': 'Postal Address For Contract', 'verbose_name_plural': 'Postal Address For Contracts'},
        ),
        migrations.AlterModelOptions(
            name='salesdocument',
            options={'verbose_name': 'Sales Document', 'verbose_name_plural': 'Sales Documents'},
        ),
        migrations.AlterModelOptions(
            name='salesdocumentposition',
            options={'verbose_name': 'Position in Sales Document', 'verbose_name_plural': 'Positions Sales Document'},
        ),
        migrations.AlterField(
            model_name='position',
            name='last_calculated_price',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Last Calculated Price'),
        ),
        migrations.AlterField(
            model_name='position',
            name='last_calculated_tax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Last Calculated Tax'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-07-27 16:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0042_auto_20180724_1816'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReportingPeriodStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=250, verbose_name='Title')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Text')),
                ('is_done', models.BooleanField(verbose_name='Status represents reporting period is closed')),
            ],
            options={
                'verbose_name': 'Reporting Period Status',
                'verbose_name_plural': 'Reporting Period Status',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-16 20:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0020_auto_20180116_2056'),
    ]

    operations = [
        migrations.CreateModel(
            name='PurchaseOrder',
            fields=[
                ('salesdocument_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.SalesDocument')),
                ('status', models.CharField(choices=[('O', 'Ordered'), ('D', 'Delayed'), ('Y', 'Delivered'), ('I', 'Invoice registered'), ('P', 'Invoice payed')], max_length=1)),
                ('supplier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Supplier', verbose_name='Supplier')),
            ],
            options={
                'verbose_name': 'Payment Reminder',
                'verbose_name_plural': 'Payment Reminders',
            },
            bases=('crm.salesdocument',),
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import CustomerGroup


class StandardCustomerGroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CustomerGroup
        django_get_or_create = ('name',)

    name = factory.Sequence(lambda n: "Customer Group #%s" % n)


class AdvancedCustomerGroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = CustomerGroup

    name = factory.Sequence(lambda n: "Customer Group #%s" % n)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-05 17:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('crm', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('accounting', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='booking',
            name='bookingReference',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='crm.Invoice', verbose_name='Booking Reference'),
        ),
        migrations.AddField(
            model_name='booking',
            name='fromAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_fromaccount',
                                    to='accounting.Account', verbose_name='From Account'),
        ),
        migrations.AddField(
            model_name='booking',
            name='lastmodifiedby',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_booking_lstmodified', to=settings.AUTH_USER_MODEL,
                                    verbose_name='Last modified by'),
        ),
        migrations.AddField(
            model_name='booking',
            name='staff',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='db_booking_refstaff', to=settings.AUTH_USER_MODEL,
                                    verbose_name='Reference Staff'),
        ),
        migrations.AddField(
            model_name='booking',
            name='toAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_toaccount',
                                    to='accounting.Account', verbose_name='To Account'),
        ),
    ]



# -*- coding: utf-8 -*-

import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_sales_document_position import StandardSalesDocumentPositionFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_quote import StandardQuoteFactory
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory
from koalixcrm.crm.factories.factory_product_price import StandardPriceFactory
from koalixcrm.crm.models import Task
from koalixcrm.crm.views.create_task import CreateTaskView


class DocumentCalculationsTest(TestCase):
    def setUp(self):
        self.test_currency = StandardCurrencyFactory.create()
        self.test_quote = StandardQuoteFactory.create()
        self.test_user = StaffUserFactory.create()
        for i in range(10):
            test_product = StandardProductTypeFactory.create(
                description="This is a test product " + i.__str__(),
                title="This is a test product " + i.__str__(),
                product_type_identifier=12334235+i,
            )
            StandardPriceFactory.create(
                product_type=test_product,
            )
            StandardSalesDocumentPositionFactory.create(
                sales_document=self.test_quote,
                position_number=i*10,
                quantity=0.333*i,
                description="This is a test position " + i.__str__(),
                discount=i*5
            )

    @pytest.mark.back_end_tests
    def test_create_task(self):
        project = CreateTaskView.create_project_from_document(self.test_user, self.test_quote)
        tasks = Task.objects.filter(project=project.id)
        task_counter = 0
        for task_current in tasks:
            self.assertEqual(
                task_current.title.__str__()[:24], "This is a test position ")
            task_counter += 1
        self.assertEqual(
            task_counter.__str__(), "10")




# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class Tax(models.Model):
    tax_rate = models.DecimalField(max_digits=5,
                                   decimal_places=2,
                                   verbose_name=_("Taxrate in Percentage"))
    name = models.CharField(verbose_name=_("Taxname"),
                            max_length=100)
    account_activa = models.ForeignKey('accounting.Account',
                                       on_delete=models.CASCADE,
                                       verbose_name=_("Activa Account"),
                                       related_name="db_relaccountactiva",
                                       null=True,
                                       blank=True)
    account_passiva = models.ForeignKey('accounting.Account',
                                        on_delete=models.CASCADE,
                                        verbose_name=_("Passiva Account"),
                                        related_name="db_relaccountpassiva",
                                        null=True,
                                        blank=True)

    def get_tax_rate(self):
        return self.tax_rate;

    def __str__(self):
        return self.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Tax')
        verbose_name_plural = _('Taxes')


class OptionTax(admin.ModelAdmin):
    list_display = ('id',
                    'tax_rate',
                    'name',
                    'account_activa',
                    'account_passiva')
    fieldsets = (('', {'fields': ('tax_rate',
                                  'name',
                                  'account_activa',
                                  'account_passiva')}),)
    allow_add = True





# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import ProductType
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_tax import StandardTaxFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardProductTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProductType
        django_get_or_create = ('product_type_identifier',)

    title = "This is a test Product"
    description = "This is a test Product"
    product_type_identifier = "123456"
    default_unit = factory.SubFactory(StandardUnitFactory)
    date_of_creation = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    last_modification = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    last_modified_by = factory.SubFactory(StaffUserFactory)
    tax = factory.SubFactory(StandardTaxFactory)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-12-10 18:05
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0003_auto_20171110_1732'),
        ('crm', '0005_auto_20171110_1732'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeliveryNote',
            fields=[
                ('salescontract_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.SalesContract')),
                ('status', models.CharField(choices=[('C', 'Created'), ('S', 'Sent'), ('R', 'Received'), ('R', 'Lost')], max_length=1)),
                ('derived_from_invoice', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Invoice')),
            ],
            options={
                'verbose_name_plural': 'Delivery Notes',
                'verbose_name': 'Delivery Note',
            },
            bases=('crm.salescontract',),
        ),
        migrations.CreateModel(
            name='PaymentReminder',
            fields=[
                ('salescontract_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.SalesContract')),
                ('iteration_number', models.IntegerField(validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(3)], verbose_name='Iteration Number')),
                ('status', models.CharField(choices=[('P', 'Payed'), ('C', 'Invoice created'), ('I', 'Invoice sent'), ('F', 'First reminder sent'), ('R', 'Second reminder sent'), ('U', 'Customer cant pay'), ('D', 'Deleted')], max_length=1)),
                ('derived_from_invoice', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Invoice')),
            ],
            options={
                'verbose_name_plural': 'Payment Reminders',
                'verbose_name': 'Payment Reminder',
            },
            bases=('crm.salescontract',),
        ),
        migrations.CreateModel(
            name='PurchaseConfirmation',
            fields=[
                ('salescontract_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.SalesContract')),
                ('validuntil', models.DateField(verbose_name='Valid until')),
                ('status', models.CharField(choices=[('S', 'Success'), ('I', 'Quote created'), ('Q', 'Quote sent'), ('F', 'First reminder sent'), ('R', 'Second reminder sent'), ('D', 'Deleted')], max_length=1, verbose_name='Status')),
            ],
            options={
                'verbose_name_plural': 'Purchase Confirmations',
                'verbose_name': 'Purchase Confirmation',
            },
            bases=('crm.salescontract',),
        ),
        migrations.CreateModel(
            name='TextParagraphInSalesContract',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('purpose', models.CharField(choices=[('BS', 'Before subject'), ('AS', 'After subject'), ('BT', 'Before total'), ('AT', 'After total'), ('BW', 'Before wishes'), ('AW', 'After wishes'), ('C1', 'Custom 1'), ('C2', 'Custom 2'), ('C3', 'Custom 3'), ('C4', 'Custom 4')], max_length=1, verbose_name='Purpose')),
                ('text_paragraph', models.TextField(verbose_name='Text')),
            ],
            options={
                'verbose_name_plural': 'TextParagraphsInSalesContract',
                'verbose_name': 'TextParagraphInSalesContract',
            },
        ),
        migrations.AlterModelOptions(
            name='supplier',
            options={'verbose_name': 'Supplier', 'verbose_name_plural': 'Suppliers'},
        ),
        migrations.RemoveField(
            model_name='position',
            name='shipmentID',
        ),
        migrations.RemoveField(
            model_name='position',
            name='supplier',
        ),
        migrations.AddField(
            model_name='salescontract',
            name='template_set',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.TemplateSet', verbose_name='Referred Template Set'),
        ),
        migrations.AddField(
            model_name='textparagraphinsalescontract',
            name='SalesContract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.SalesContract'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.http import Http404
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _
from django.contrib.contenttypes.models import ContentType
from koalixcrm.crm.exceptions import *
from koalixcrm.djangoUserExtension.exceptions import *
from koalixcrm.crm.documents.sales_document import SalesDocument
from koalixcrm.crm.documents.sales_document_position import SalesDocumentPosition
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.reporting.generic_task_link import GenericTaskLink
from koalixcrm.crm.reporting.project import Project
from koalixcrm.global_support_functions import *
from datetime import date


class CreateTaskView:

    @staticmethod
    def create_task_from_sales_document_position(sales_document_position,
                                                 user,
                                                 document,
                                                 project):
        date_now = date.today()
        content_type_sales_document_position = ContentType.objects.get_for_model(SalesDocumentPosition)
        task_title = limit_string_length(sales_document_position.description, 30)
        try:
            existing_task = GenericTaskLink.objects.get(content_type=content_type_sales_document_position,
                                                        object_id=sales_document_position.id)
            task_id = existing_task.task.id
            task = Task.objects.filter(id=task_id).update(
                title=task_title,
                planned_start_date=date_now,
                reporting_period=project,
                description=sales_document_position.description,
                last_status_change=date_now
            )
        except ObjectDoesNotExist:
            task = Task.objects.create(
                title=task_title,
                project=project,
                description=sales_document_position.description,
                last_status_change=date_now
            )
            GenericTaskLink.objects.create(
                task=task,
                content_type=content_type_sales_document_position,
                object_id=sales_document_position.id,
                last_modified_by=user
            )
            GenericTaskLink.objects.create(
                    task=task,
                    content_type=ContentType.objects.get_for_model(SalesDocument),
                    object_id=document.id,
                    last_modified_by=user
                )
        return task

    @staticmethod
    def create_project_from_document(user, document):
        sales_document_positions = SalesDocumentPosition.objects.filter(sales_document=document)
        project_name = limit_string_length(document.contract.description, 30)
        project = Project.objects.create(project_manager=user,
                                         project_name=project_name,
                                         description=document.contract.description,
                                         default_template_set=document.contract.default_template_set,
                                         date_of_creation=date.today(),
                                         last_modification=date.today(),
                                         last_modified_by=user,
                                         default_currency=document.currency)
        for sales_document_position in sales_document_positions:
            CreateTaskView.create_task_from_sales_document_position(sales_document_position,
                                                                    user,
                                                                    document,
                                                                    project)
        return project

    @staticmethod
    def create_project(calling_model_admin, request, document, redirect_to):
        """This method creates tasks from the positions of a sales document

            Args:
              calling_model_admin (ModelAdmin):  The calling ModelAdmin must be provided for error message response.
              request: The request User is to know where to save the error message
              document (SalesDocument):  The model from which a tasks shall be created
              redirect_to (str): String that describes to where the method should redirect in case of an error

            Returns:
              HTTpResponseRedirect when not successful

            Raises:
              raises Http404 exception if anything goes wrong"""
        try:
            project = CreateTaskView.create_project_from_document(request.user, document)
            calling_model_admin.message_user(request, _("Successfully created Project and Tasks for this contract"))
            response = HttpResponseRedirect('/admin/crm/' +
                                            project.__class__.__name__.lower() +
                                            '/' +
                                            str(project.id))
        except (TemplateSetMissing,
                UserExtensionMissing,
                UserExtensionEmailAddressMissing,
                UserExtensionPhoneAddressMissing) as e:
            if isinstance(e, UserExtensionMissing):
                return render(request, 'crm/admin/exception.html')
            elif isinstance(e, UserExtensionEmailAddressMissing):
                return render(request, 'crm/admin/exception.html')
            else:
                raise Http404
        return response



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _

from koalixcrm.crm.contact.postal_address import PostalAddress
from koalixcrm.crm.contact.phone_address import PhoneAddress
from koalixcrm.crm.contact.email_address import EmailAddress
from koalixcrm.djangoUserExtension.const.purpose import *
from koalixcrm.djangoUserExtension.exceptions import *
from koalixcrm.global_support_functions import xstr


class UserExtension(models.Model):
    user = models.ForeignKey("auth.User",
                             on_delete=models.CASCADE,
                             blank=False,
                             null=False)
    default_template_set = models.ForeignKey("TemplateSet", on_delete=models.CASCADE)
    default_currency = models.ForeignKey("crm.Currency", on_delete=models.CASCADE)

    @staticmethod
    def objects_to_serialize(object_to_create_pdf, reference_user):
        from koalixcrm.crm.contact.phone_address import PhoneAddress
        from koalixcrm.crm.contact.email_address import EmailAddress
        from django.contrib import auth
        objects = list(auth.models.User.objects.filter(id=reference_user.id))
        user_extension = UserExtension.objects.filter(user=reference_user.id)
        if len(user_extension) == 0:
            raise UserExtensionMissing(_("During "+str(object_to_create_pdf)+" PDF Export"))
        phone_address = UserExtensionPhoneAddress.objects.filter(
            userExtension=user_extension[0].id)
        if len(phone_address) == 0:
            raise UserExtensionPhoneAddressMissing(_("During "+str(object_to_create_pdf)+" PDF Export"))
        email_address = UserExtensionEmailAddress.objects.filter(
            userExtension=user_extension[0].id)
        if len(email_address) == 0:
            raise UserExtensionEmailAddressMissing(_("During "+str(object_to_create_pdf)+" PDF Export"))
        objects += list(user_extension)
        objects += list(EmailAddress.objects.filter(id=email_address[0].id))
        objects += list(PhoneAddress.objects.filter(id=phone_address[0].id))
        return objects

    @staticmethod
    def get_user_extension(django_user):
        user_extensions = UserExtension.objects.filter(user=django_user)
        if len(user_extensions) > 1:
            raise TooManyUserExtensionsAvailable(_("More than one User Extension define for user ") + django_user.__str__())
        elif len(user_extensions) == 0:
            raise UserExtensionMissing(_("No User Extension define for user ") + django_user.__str__())
        return user_extensions[0]

    def get_template_set(self, template_set):
        if template_set == self.default_template_set.work_report_template:
            if self.default_template_set.work_report_template:
                return self.default_template_set.work_report_template
            else:
                raise TemplateSetMissingForUserExtension((_("Template Set for work report " +
                                                            "is missing for User Extension" + str(self))))

    def get_fop_config_file(self, template_set):
        template_set = self.get_template_set(template_set)
        return template_set.get_fop_config_file()

    def get_xsl_file(self, template_set):
        template_set = self.get_template_set(template_set)
        return template_set.get_xsl_file()

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('User Extension')
        verbose_name_plural = _('User Extension')

    def __str__(self):
        return xstr(self.id) + ' ' + xstr(self.user.__str__())


class UserExtensionPostalAddress(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension, on_delete=models.CASCADE)

    def __str__(self):
        return xstr(self.name) + ' ' + xstr(self.pre_name)

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('Postal Address for User Extension')
        verbose_name_plural = _('Postal Address for User Extension')


class UserExtensionPhoneAddress(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension, on_delete=models.CASCADE)

    def __str__(self):
        return xstr(self.phone)

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('Phone number for User Extension')
        verbose_name_plural = _('Phone number for User Extension')


class UserExtensionEmailAddress(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension, on_delete=models.CASCADE)

    def __str__(self):
        return xstr(self.email)

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('Email Address for User Extension')
        verbose_name_plural = _('Email Address for User Extension')


class InlineUserExtensionPostalAddress(admin.StackedInline):
    model = UserExtensionPostalAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': (
                'prefix',
                'pre_name',
                'name',
                'address_line_1',
                'address_line_2',
                'address_line_3',
                'address_line_4',
                'zip_code',
                'town',
                'state',
                'country',
                'purpose')
        }),
    )
    allow_add = True


class InlineUserExtensionPhoneAddress(admin.StackedInline):
    model = UserExtensionPhoneAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('phone',
                       'purpose',)
        }),
    )
    allow_add = True


class InlineUserExtensionEmailAddress(admin.StackedInline):
    model = UserExtensionEmailAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('email',
                       'purpose',)
        }),
    )
    allow_add = True


class OptionUserExtension(admin.ModelAdmin):
    list_display = ('id',
                    'user',
                    'default_template_set',
                    'default_currency')
    list_display_links = ('id',
                          'user')
    list_filter = ('user',
                   'default_template_set',)
    ordering = ('id',)
    search_fields = ('id',
                     'user')
    fieldsets = (
        (_('Basics'), {
            'fields': ('user',
                       'default_template_set',
                       'default_currency')
        }),
    )

    def create_work_report_pdf(self, request, queryset):
        from koalixcrm.crm.views.create_work_report import create_work_report

        return create_work_report(self, request, queryset)

    create_work_report_pdf.short_description = _("Work Report PDF")

    save_as = True
    actions = [create_work_report_pdf]
    inlines = [InlineUserExtensionPostalAddress,
               InlineUserExtensionPhoneAddress,
               InlineUserExtensionEmailAddress]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-09 19:49
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0002_auto_20170927_2042'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='contact',
            options={'verbose_name': 'Kontakt', 'verbose_name_plural': 'Kontakt'},
        ),
        migrations.AlterModelOptions(
            name='contract',
            options={'verbose_name': 'Auftrag', 'verbose_name_plural': 'Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='currency',
            options={'verbose_name': 'Währung', 'verbose_name_plural': 'Währung'},
        ),
        migrations.AlterModelOptions(
            name='customer',
            options={'verbose_name': 'Kunde', 'verbose_name_plural': 'Kunden'},
        ),
        migrations.AlterModelOptions(
            name='customerbillingcycle',
            options={'verbose_name': 'Zahlungsbedingungen', 'verbose_name_plural': 'Zahlungsbedingungen'},
        ),
        migrations.AlterModelOptions(
            name='customergroup',
            options={'verbose_name': 'Kundengruppe', 'verbose_name_plural': 'Kundengruppen'},
        ),
        migrations.AlterModelOptions(
            name='customergrouptransform',
            options={'verbose_name': 'Kundengruppe Preisumrechnung', 'verbose_name_plural': 'Kundengruppen Preisumrechnungen'},
        ),
        migrations.AlterModelOptions(
            name='emailaddress',
            options={'verbose_name': 'Email Adresse', 'verbose_name_plural': 'Email Adresse'},
        ),
        migrations.AlterModelOptions(
            name='emailaddressforcontact',
            options={'verbose_name': 'Email Adresse für Kontakte', 'verbose_name_plural': 'Email Adresse für Kontakte'},
        ),
        migrations.AlterModelOptions(
            name='emailaddressforcontract',
            options={'verbose_name': 'Email Adresse für Aufträge', 'verbose_name_plural': 'Email Adresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='emailaddressforpurchaseorder',
            options={'verbose_name': 'Email Adresse für Aufträge', 'verbose_name_plural': 'Email Adresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='emailaddressforsalescontract',
            options={'verbose_name': 'Email Adresse für Aufträge', 'verbose_name_plural': 'Email Adresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='invoice',
            options={'verbose_name': 'Rechnung', 'verbose_name_plural': 'Rechnungen'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddress',
            options={'verbose_name': 'Telefonnummer', 'verbose_name_plural': 'Telefonnummer'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforcontact',
            options={'verbose_name': 'Telefonnummer für Kontakte', 'verbose_name_plural': 'Telefonnummer für Kontakte'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforcontract',
            options={'verbose_name': 'Telefonnummer für Aufträge', 'verbose_name_plural': 'Telefonnummer für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforpurchaseorder',
            options={'verbose_name': 'Telefonnummer für Aufträge', 'verbose_name_plural': 'Telefonnummer für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforsalescontract',
            options={'verbose_name': 'Telefonnummer für Aufträge', 'verbose_name_plural': 'Telefonnummer für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='position',
            options={'verbose_name': 'Position', 'verbose_name_plural': 'Positionen'},
        ),
        migrations.AlterModelOptions(
            name='postaladdress',
            options={'verbose_name': 'Postadresse', 'verbose_name_plural': 'Postadresse'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforcontact',
            options={'verbose_name': 'Postadresse für Kontakte', 'verbose_name_plural': 'Postadresse für Kontakte'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforcontract',
            options={'verbose_name': 'Postadresse für Aufträge', 'verbose_name_plural': 'Postadresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforpurchaseorder',
            options={'verbose_name': 'Postadresse für Aufträge', 'verbose_name_plural': 'Postadresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforsalescontract',
            options={'verbose_name': 'Postadresse für Aufträge', 'verbose_name_plural': 'Postadresse für Aufträge'},
        ),
        migrations.AlterModelOptions(
            name='price',
            options={'verbose_name': 'Preis', 'verbose_name_plural': 'Preise'},
        ),
        migrations.AlterModelOptions(
            name='product',
            options={'verbose_name': 'Produkt', 'verbose_name_plural': 'Produkte'},
        ),
        migrations.AlterModelOptions(
            name='purchaseorder',
            options={'verbose_name': 'Bestellung', 'verbose_name_plural': 'Bestellung'},
        ),
        migrations.AlterModelOptions(
            name='purchaseorderposition',
            options={'verbose_name': 'Bestellung Position', 'verbose_name_plural': 'Bestellung Positionen'},
        ),
        migrations.AlterModelOptions(
            name='quote',
            options={'verbose_name': 'Offerte', 'verbose_name_plural': 'Offerten'},
        ),
        migrations.AlterModelOptions(
            name='salescontract',
            options={'verbose_name': 'Verkaufsauftrag', 'verbose_name_plural': 'Verkaufsaufträge'},
        ),
        migrations.AlterModelOptions(
            name='salescontractposition',
            options={'verbose_name': 'Verkaufsaufträg Position', 'verbose_name_plural': 'Verkaufsaufträge Positionen'},
        ),
        migrations.AlterModelOptions(
            name='supplier',
            options={'verbose_name': 'Lieferant', 'verbose_name_plural': 'Lieferant'},
        ),
        migrations.AlterModelOptions(
            name='tax',
            options={'verbose_name': 'Steuer', 'verbose_name_plural': 'Steuern'},
        ),
        migrations.AlterModelOptions(
            name='unit',
            options={'verbose_name': 'Masseinheit', 'verbose_name_plural': 'Masseinheiten'},
        ),
        migrations.AlterModelOptions(
            name='unittransform',
            options={'verbose_name': 'Einheiten Umrechnung', 'verbose_name_plural': 'Einheiten Umrechnungen'},
        ),
        migrations.AddField(
            model_name='purchaseorder',
            name='last_print_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Last printed'),
        ),
        migrations.AddField(
            model_name='salescontract',
            name='last_print_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Last printed'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='lastmodifiedby',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='name',
            field=models.CharField(max_length=300, verbose_name='Dateiname'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='defaultSupplier',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Supplier', verbose_name='Standard Lieferant'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='defaultcurrency',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Currency', verbose_name='Standard Währung'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='defaultcustomer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Customer', verbose_name='Standard Kunde'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='description',
            field=models.TextField(verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='lastmodifiedby',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_contractlstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='staff',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relcontractstaff', to=settings.AUTH_USER_MODEL, verbose_name='Mitarbeiter'),
        ),
        migrations.AlterField(
            model_name='currency',
            name='description',
            field=models.CharField(max_length=100, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='currency',
            name='rounding',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Rundung'),
        ),
        migrations.AlterField(
            model_name='currency',
            name='shortName',
            field=models.CharField(max_length=3, verbose_name='Angezeigt nach dem Preis in der Position'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='defaultCustomerBillingCycle',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.CustomerBillingCycle', verbose_name='Standard Zahlungsbedingung'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='ismemberof',
            field=models.ManyToManyField(blank=True, to='crm.CustomerGroup', verbose_name='Ist Mitglied von'),
        ),
        migrations.AlterField(
            model_name='customerbillingcycle',
            name='name',
            field=models.CharField(max_length=300, verbose_name='Dateiname'),
        ),
        migrations.AlterField(
            model_name='customerbillingcycle',
            name='timeToPaymentDate',
            field=models.IntegerField(verbose_name='Tage bis zur Fälligkeit'),
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='factor',
            field=models.IntegerField(blank=True, null=True, verbose_name='Prismultiplikator zwischen der einen und der anderen Kundengruppe'),
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='fromCustomerGroup',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromfromcustomergroup', to='crm.CustomerGroup', verbose_name='Von Masseinheit'),
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Product', verbose_name='Produkt'),
        ),
        migrations.AlterField(
            model_name='customergrouptransform',
            name='toCustomerGroup',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromtocustomergroup', to='crm.CustomerGroup', verbose_name='Zu Masseinheit'),
        ),
        migrations.AlterField(
            model_name='emailaddress',
            name='email',
            field=models.EmailField(max_length=200, verbose_name='Email Adresse'),
        ),
        migrations.AlterField(
            model_name='emailaddressforcontact',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='emailaddressforcontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='emailaddressforpurchaseorder',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='emailaddressforsalescontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='payableuntil',
            field=models.DateField(verbose_name='Zahlbar bis'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='paymentBankReference',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Zahlungsreferenz der Bank'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='status',
            field=models.CharField(choices=[('P', 'Bezahlt'), ('C', 'Rechnung erstellt'), ('I', 'Rechung geschickt'), ('F', 'Erste Erinnerung geschickt'), ('R', 'Zweite Erinnerung geschickt'), ('U', 'Kunde kann nicht zahlen'), ('D', 'Gelöscht')], max_length=1),
        ),
        migrations.AlterField(
            model_name='phoneaddress',
            name='phone',
            field=models.CharField(max_length=20, verbose_name='Telefonnummer'),
        ),
        migrations.AlterField(
            model_name='phoneaddressforcontact',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='phoneaddressforcontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='phoneaddressforpurchaseorder',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='phoneaddressforsalescontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='position',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='position',
            name='discount',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Rabatt'),
        ),
        migrations.AlterField(
            model_name='position',
            name='lastCalculatedPrice',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Zuletzt berechneter Preis'),
        ),
        migrations.AlterField(
            model_name='position',
            name='lastCalculatedTax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Zuletzt berechnete Steuer'),
        ),
        migrations.AlterField(
            model_name='position',
            name='lastPricingDate',
            field=models.DateField(blank=True, null=True, verbose_name='Datum der letze Preisberechnung'),
        ),
        migrations.AlterField(
            model_name='position',
            name='overwriteProductPrice',
            field=models.BooleanField(verbose_name='Überschreie Produkt Preis'),
        ),
        migrations.AlterField(
            model_name='position',
            name='positionNumber',
            field=models.IntegerField(verbose_name='Positionsnummer'),
        ),
        migrations.AlterField(
            model_name='position',
            name='positionPricePerUnit',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Preis pro Einheit'),
        ),
        migrations.AlterField(
            model_name='position',
            name='product',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Product', verbose_name='Produkt'),
        ),
        migrations.AlterField(
            model_name='position',
            name='quantity',
            field=models.DecimalField(decimal_places=3, max_digits=10, verbose_name='Anzahl'),
        ),
        migrations.AlterField(
            model_name='position',
            name='sentOn',
            field=models.DateField(blank=True, null=True, verbose_name='Verschickt am'),
        ),
        migrations.AlterField(
            model_name='position',
            name='shipmentID',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Lieferungs ID'),
        ),
        migrations.AlterField(
            model_name='position',
            name='supplier',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Supplier', verbose_name='Versandunternehmen'),
        ),
        migrations.AlterField(
            model_name='position',
            name='unit',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Unit', verbose_name='Masseinheit'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='addressline1',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Adresslinie 1'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='addressline2',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Adresslinie 2'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='addressline3',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Adresslinie 3'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='addressline4',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Adresslinie 4'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='country',
            field=models.CharField(blank=True, choices=[('AF', 'Afghanistan'), ('AX', 'Aland Islands'), ('AL', 'Albanien'), ('DZ', 'Algerien'), ('AS', 'Amerikanisch-Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarktis'), ('AG', 'Antigua und Barbuda'), ('AR', 'Argentinien'), ('AM', 'Armenien'), ('AW', 'Aruba'), ('AU', 'Australien'), ('AT', 'Österreich'), ('AZ', 'Aserbaidschan'), ('BS', 'Bahamas'), ('BH', 'Bahrain\t'), ('BD', 'Bangladesch'), ('BB', 'Barbados'), ('BY', 'Belarus, (Weißrußland)'), ('BE', 'Belgien'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivien'), ('BA', 'Bosnien und Herzegovina'), ('BW', 'Botsuana'), ('BV', 'Bouvetinsel'), ('BR', 'Brasilien'), ('IO', 'Britisches Territorium im Indischen Ozean'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgarien'), ('BF', 'Burkina Faso (Obervolta - 1984)'), ('BI', 'Burundi'), ('KH', 'Kambodscha'), ('CM', 'Kamerun'), ('CA', 'Kanada'), ('CV', 'Kap Verde'), ('KY', 'Kaimaninseln'), ('CF', 'Zentralafrikanische Republik'), ('TD', 'Tschad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Weihnachtsinsel'), ('CC', 'Kokosinseln (Keeling)'), ('CO', 'Kolumbien'), ('KM', 'Komoren'), ('CG', 'Kongo'), ('CD', 'Kongo, Demokratische Republik'), ('CK', 'Cookinseln'), ('CR', 'Costa Rica'), ('CI', 'Elfenbeinküste'), ('HR', 'Kroatien'), ('CU', 'Kuba'), ('CY', 'Zypern'), ('CZ', 'Tschechische Republik'), ('DK', 'Dänemark'), ('DJ', 'Dschibuti'), ('DM', 'Dominica'), ('DO', 'Dominikanische Republik (Dom Rep)'), ('EC', 'Ecuador'), ('EG', 'Ägypten'), ('SV', 'El Salvador'), ('GQ', 'Äquatorialguinea'), ('ER', 'Eritrea'), ('EE', 'Estland'), ('ET', 'Äthiopien'), ('FK', 'Falklandinseln (Malvinen)'), ('FO', 'Färöer'), ('FJ', 'Fidschi'), ('FI', 'Finnland'), ('FR', 'Frankreich'), ('GF', 'Französisch-Guayana'), ('PF', 'Französisch-Polynesien'), ('TF', 'Französische Süd- und Antarktisgebiete'), ('GA', 'Gabun'), ('GM', 'Gambia'), ('GE', 'Georgien'), ('DE', 'Deutschland'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Griechenland'), ('GL', 'Grönland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard und die Mcdonaldinseln'), ('VA', 'Heiliger Stuhl (Vatikanstadt)'), ('HN', 'Honduras'), ('HK', 'Hongkong'), ('HU', 'Ungarn'), ('IS', 'Island'), ('IN', 'Indien'), ('ID', 'Indonesien'), ('IR', 'Iran, Islamische Republik'), ('IQ', 'Irak'), ('IE', 'Irland'), ('IM', 'Insel Man'), ('IL', 'Israel'), ('IT', 'Italien'), ('JM', 'Jamaika'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordanien'), ('KZ', 'Kasachstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', 'Korea, Demokratische Volksrepublik (Nord Korea)'), ('KR', 'Korea, Republik (Süd Korea)'), ('KW', 'Kuwait'), ('KG', 'Kirgisistan'), ('LA', 'Laos, Demokratische Volksrepublik'), ('LV', 'Lettland'), ('LB', 'Libanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libyen, Libysch-Arabische Dschamahirija'), ('LI', 'Liechtenstein'), ('LT', 'Litauen'), ('LU', 'Luxemburg'), ('MO', 'Macau'), ('MK', 'Mazedonien'), ('MG', 'Madagaskar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Malediven'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshallinseln'), ('MQ', 'Martinique'), ('MR', 'Mauretanien'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexiko'), ('FM', 'Mikronesien, Föderierte Staaten von'), ('MD', 'Moldau, Republik'), ('MC', 'Monaco'), ('MN', 'Mongolei'), ('ME', 'Montenegro'), ('MS', 'Montenegro'), ('MA', 'Marokko'), ('MZ', 'Mosambik'), ('MM', 'Myanmar, Birma'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Niederlande'), ('AN', 'Niederländische Antillen'), ('NC', 'Neukaledonien'), ('NZ', 'Neuseeland'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolkinsel'), ('MP', 'Marianen, Nördliche'), ('NO', 'Norwegen'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palästinensische Autonomiegebiete'), ('PA', 'Panama'), ('PG', 'Papua-Neuguinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippinen'), ('PN', 'Pitcairninseln'), ('PL', 'Polen'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Katar'), ('RE', 'Reunion'), ('RO', 'Rumänien'), ('RU', 'Russische Föderation (Russland)'), ('RW', 'Ruanda'), ('SH', 'St. Helena'), ('KN', 'St. Kitts und Nevis'), ('LC', 'St. Lucia'), ('PM', 'St. Pierre und Miquelon'), ('VC', 'St. Vincent und die Grenadinen'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'São Tomé und Príncipe'), ('SA', 'Saudi-Arabien'), ('SN', 'Senegal'), ('RS', 'Serbien'), ('SC', 'Seychellen'), ('SL', 'Sierra Leone'), ('SG', 'Singapur'), ('SK', 'Slowakei'), ('SI', 'Slowenien'), ('SB', 'Salomonen'), ('SO', 'Somalia'), ('ZA', 'Südafrika'), ('GS', 'Südgeorgien und die südlichen Sandwichinseln'), ('ES', 'Spanien'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard und Jan Mayen'), ('SZ', 'Swasiland'), ('SE', 'Schweden'), ('CH', 'Schweiz'), ('SY', 'Syrien, Arabische Republik'), ('TW', 'Taiwan'), ('TJ', 'Tadschikistan'), ('TZ', 'Tansania, Vereinigte Republik'), ('TH', 'Thailand'), ('TL', 'Timor-Leste (Osttimor)'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad und Tobago'), ('TN', 'Tunesien'), ('TR', 'Türkei'), ('TM', 'Turkmenistan'), ('TC', 'Turks- und Caicosinseln'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'Vereinigte Arabische Emirate'), ('GB', 'Vereinigtes Königreich'), ('US', 'Vereinigte Staaten'), ('UM', 'Amerikanische Überseeinseln, Kleinere'), ('UY', 'Uruguay'), ('UZ', 'Usbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela'), ('VN', 'Vietnam'), ('VG', 'Britische Jungferninseln'), ('VI', 'Amerikanische Jungferninseln'), ('WF', 'Wallis und Futuna'), ('EH', 'Westsahara'), ('YE', 'Jemen'), ('ZM', 'Sambia'), ('ZW', 'Simbabwe')], max_length=2, null=True, verbose_name='Land'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='name',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Dateiname'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='prefix',
            field=models.CharField(blank=True, choices=[('F', 'Firma'), ('W', 'Frau'), ('H', 'Herr'), ('G', 'Frau')], max_length=1, null=True, verbose_name='Anrede'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='prename',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Vorname'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='state',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Bundestaat/Kanton'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='town',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Ortschaft'),
        ),
        migrations.AlterField(
            model_name='postaladdress',
            name='zipcode',
            field=models.IntegerField(blank=True, null=True, verbose_name='PLZ'),
        ),
        migrations.AlterField(
            model_name='postaladdressforcontact',
            name='purpose',
            field=models.CharField(choices=[('H', 'Privat'), ('O', 'Geschäftlich'), ('P', 'Mobil Privat'), ('B', 'Mobil Geschäftlich')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='postaladdressforcontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='postaladdressforpurchaseorder',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='postaladdressforsalescontract',
            name='purpose',
            field=models.CharField(choices=[('D', 'Versand Addresse'), ('B', 'Rechnungs Adresse'), ('C', 'Kontakt Adresse')], max_length=1, verbose_name='Verwendungszweck'),
        ),
        migrations.AlterField(
            model_name='price',
            name='customerGroup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.CustomerGroup', verbose_name='Kundengruppe'),
        ),
        migrations.AlterField(
            model_name='price',
            name='price',
            field=models.DecimalField(decimal_places=2, max_digits=17, verbose_name='Preis pro Einheit'),
        ),
        migrations.AlterField(
            model_name='price',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Product', verbose_name='Produkt'),
        ),
        migrations.AlterField(
            model_name='price',
            name='unit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Unit', verbose_name='Masseinheit'),
        ),
        migrations.AlterField(
            model_name='price',
            name='validfrom',
            field=models.DateField(blank=True, null=True, verbose_name='Gültig vom'),
        ),
        migrations.AlterField(
            model_name='price',
            name='validuntil',
            field=models.DateField(blank=True, null=True, verbose_name='Gültig bis'),
        ),
        migrations.AlterField(
            model_name='product',
            name='accoutingProductCategorie',
            field=models.ForeignKey(blank='True', null=True, on_delete=django.db.models.deletion.CASCADE, to='accounting.ProductCategorie', verbose_name='Buchhaltung Produktkategorie'),
        ),
        migrations.AlterField(
            model_name='product',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='product',
            name='defaultunit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Unit', verbose_name='Masseinheit'),
        ),
        migrations.AlterField(
            model_name='product',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='product',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='product',
            name='lastmodifiedby',
            field=models.ForeignKey(blank='True', null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='product',
            name='productNumber',
            field=models.IntegerField(verbose_name='Produktnummer'),
        ),
        migrations.AlterField(
            model_name='product',
            name='title',
            field=models.CharField(max_length=200, verbose_name='Titel'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract', verbose_name='Auftrag'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='currency',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Currency', verbose_name='Währung'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='description',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='externalReference',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Externe Referenz'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastCalculatedPrice',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Zuletzt berechneter Preis mit Steuer'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastCalculatedTax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Zuletzt berechnete Steuer'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastPricingDate',
            field=models.DateField(blank=True, null=True, verbose_name='Datum der letze Preisberechnung'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='lastmodifiedby',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_polstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='staff',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relpostaff', to=settings.AUTH_USER_MODEL, verbose_name='Mitarbeiter'),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='status',
            field=models.CharField(choices=[('O', 'Bestellt'), ('D', 'Verspätet'), ('Y', 'Ausgeliefert'), ('I', 'Rechnung registriert'), ('P', 'Rechnung bezahlt')], max_length=1),
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='supplier',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Supplier', verbose_name='Lieferant'),
        ),
        migrations.AlterField(
            model_name='purchaseorderposition',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.PurchaseOrder', verbose_name='Auftrag'),
        ),
        migrations.AlterField(
            model_name='quote',
            name='status',
            field=models.CharField(choices=[('S', 'Erfolg'), ('I', 'Offerte erstellt'), ('Q', 'Offerte verschickt'), ('F', 'Erste Erinnerung geschickt'), ('R', 'Zweite Erinnerung geschickt'), ('D', 'Gelöscht')], max_length=1, verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='quote',
            name='validuntil',
            field=models.DateField(verbose_name='Gültig bis'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Contract', verbose_name='Auftrag'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='currency',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Currency', verbose_name='Währung'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Customer', verbose_name='Kunde'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='description',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='discount',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='Rabatt'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='externalReference',
            field=models.CharField(blank=True, max_length=100, verbose_name='Externe Referenz'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastCalculatedPrice',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Preis ohne Steuer'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastCalculatedTax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=17, null=True, verbose_name='Steuer'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastPricingDate',
            field=models.DateField(blank=True, null=True, verbose_name='Letzte Preisberechnung'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='lastmodifiedby',
            field=models.ForeignKey(blank='True', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_lstscmodified', to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='salescontract',
            name='staff',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relscstaff', to=settings.AUTH_USER_MODEL, verbose_name='Mitarbeiter'),
        ),
        migrations.AlterField(
            model_name='salescontractposition',
            name='contract',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.SalesContract', verbose_name='Auftrag'),
        ),
        migrations.AlterField(
            model_name='tax',
            name='accountActiva',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relaccountactiva', to='accounting.Account', verbose_name='Aktiv Konto'),
        ),
        migrations.AlterField(
            model_name='tax',
            name='accountPassiva',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relaccountpassiva', to='accounting.Account', verbose_name='Passiv Konto'),
        ),
        migrations.AlterField(
            model_name='tax',
            name='name',
            field=models.CharField(max_length=100, verbose_name='Name der Steuer'),
        ),
        migrations.AlterField(
            model_name='tax',
            name='taxrate',
            field=models.DecimalField(decimal_places=2, max_digits=5, verbose_name='Steuersatz in Prozent'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='description',
            field=models.CharField(max_length=100, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='fractionFactorToNextHigherUnit',
            field=models.IntegerField(blank=True, null=True, verbose_name='Multiplikator zwischen dieser und der nächst höheren Einheit'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='isAFractionOf',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Unit', verbose_name='Ist eine Untereinheit von'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='shortName',
            field=models.CharField(max_length=3, verbose_name='Angezeigt nach der Anzahl in der Position'),
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='factor',
            field=models.IntegerField(blank=True, null=True, verbose_name='Multiplikator zwischen der einen und der anderen Einheit'),
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='fromUnit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromfromunit', to='crm.Unit', verbose_name='Von Masseinheit'),
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='crm.Product', verbose_name='Produkt'),
        ),
        migrations.AlterField(
            model_name='unittransform',
            name='toUnit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_reltransfromtounit', to='crm.Unit', verbose_name='Zu Masseinheit'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-09 19:49
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0003_remove_account_originalamount'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['accountNumber'], 'verbose_name': 'Konto', 'verbose_name_plural': 'Konto'},
        ),
        migrations.AlterModelOptions(
            name='accountingperiod',
            options={'verbose_name': 'Abrechnungszeitraum', 'verbose_name_plural': 'Abrechnungszeiträume'},
        ),
        migrations.AlterModelOptions(
            name='booking',
            options={'verbose_name': 'Buchung', 'verbose_name_plural': 'Buchungen'},
        ),
        migrations.AlterModelOptions(
            name='productcategorie',
            options={'verbose_name': 'Produktekategorie', 'verbose_name_plural': 'Produktekategorien'},
        ),
        migrations.AlterField(
            model_name='account',
            name='accountNumber',
            field=models.IntegerField(verbose_name='Kontonummer'),
        ),
        migrations.AlterField(
            model_name='account',
            name='accountType',
            field=models.CharField(choices=[('E', 'Ertrag (Einnahmen)'), ('S', 'Aufwand (Ausgaben)'), ('L', 'Passiva (Schulden)'), ('A', 'Aktiva (Vermögen)')], max_length=1, verbose_name='Kontotyp'),
        ),
        migrations.AlterField(
            model_name='account',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isACustomerPaymentAccount',
            field=models.BooleanField(verbose_name='Konto für Kundeneinzahlungen'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isProductInventoryActiva',
            field=models.BooleanField(verbose_name='Konto für Lager'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isopeninterestaccount',
            field=models.BooleanField(verbose_name='Konto für offene Forderungen'),
        ),
        migrations.AlterField(
            model_name='account',
            name='isopenreliabilitiesaccount',
            field=models.BooleanField(verbose_name='Konto für offene Rechungen'),
        ),
        migrations.AlterField(
            model_name='account',
            name='title',
            field=models.CharField(max_length=50, verbose_name='Kontobezeichnung'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='begin',
            field=models.DateField(verbose_name='Anfang'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='end',
            field=models.DateField(verbose_name='Ende'),
        ),
        migrations.AlterField(
            model_name='accountingperiod',
            name='title',
            field=models.CharField(max_length=200, verbose_name='Titel'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='accountingPeriod',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='accounting.AccountingPeriod', verbose_name='Abrechnungszeitraum'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='amount',
            field=models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Betrag'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='bookingDate',
            field=models.DateTimeField(verbose_name='Buchung am'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='bookingReference',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Invoice', verbose_name='Buchungsreferenz'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='dateofcreation',
            field=models.DateTimeField(auto_now=True, verbose_name='Erstelldatum'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='description',
            field=models.CharField(blank=True, max_length=120, null=True, verbose_name='Beschreibung'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='fromAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_fromaccount', to='accounting.Account', verbose_name='Vom Konto (Soll)'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='lastmodification',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Zuletzt geändert'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='lastmodifiedby',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_lstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Zuletzt geändert durch'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='staff',
            field=models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_refstaff', to=settings.AUTH_USER_MODEL, verbose_name='Zuständiger Mitarbeiter'),
        ),
        migrations.AlterField(
            model_name='booking',
            name='toAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_booking_toaccount', to='accounting.Account', verbose_name='Zu Konto (Haben)'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='lossAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_loss_account', to='accounting.Account', verbose_name='Aufwand Konto'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='profitAccount',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_profit_account', to='accounting.Account', verbose_name='Ertrag Konto'),
        ),
        migrations.AlterField(
            model_name='productcategorie',
            name='title',
            field=models.CharField(max_length=50, verbose_name='Produktekategorie'),
        ),
    ]



from rest_framework import serializers

from koalixcrm.crm.contact.customer_billing_cycle import CustomerBillingCycle


class OptionCustomerBillingCycleJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    name = serializers.CharField(read_only=True)

    class Meta:
        model = CustomerBillingCycle
        fields = ('id',
                  'name')


class CustomerBillingCycleJSONSerializer(serializers.HyperlinkedModelSerializer):
    daysToPaymentDate = serializers.IntegerField(source='time_to_payment_date', allow_null=False)
    paymentReminderDaysToPayment = serializers.IntegerField(source='payment_reminder_time_to_payment', allow_null=True)

    class Meta:
        model = CustomerBillingCycle
        fields = ('id',
                  'name',
                  'daysToPaymentDate',
                  'paymentReminderDaysToPayment')



from rest_framework import serializers

from koalixcrm.crm.product.currency import Currency


class CurrencyJSONSerializer(serializers.HyperlinkedModelSerializer):
    shortName = serializers.CharField(source='short_name')

    class Meta:
        model = Currency
        fields = ('id',
                  'description',
                  'shortName',
                  'rounding')



# -*- coding: utf-8 -*-

from koalixcrm.crm.contact.contact import *
from koalixcrm.crm.contact.customer_group import *
from koalixcrm.crm.contact.customer import *
from koalixcrm.crm.contact.postal_address import *
from koalixcrm.crm.contact.customer_billing_cycle import *
from koalixcrm.crm.contact.email_address import *
from koalixcrm.crm.contact.phone_address import *
from koalixcrm.crm.contact.supplier import *

from koalixcrm.crm.documents.contract import Contract, PostalAddressForContract
from koalixcrm.crm.documents.contract import PhoneAddressForContract, EmailAddressForContract
from koalixcrm.crm.documents.sales_document_position import Position, SalesDocumentPosition
from koalixcrm.crm.documents.sales_document import SalesDocument, PostalAddressForSalesDocument
from koalixcrm.crm.documents.sales_document import EmailAddressForSalesDocument, PhoneAddressForSalesDocument
from koalixcrm.crm.documents.sales_document import TextParagraphInSalesDocument
from koalixcrm.crm.documents.invoice import Invoice
from koalixcrm.crm.documents.purchase_confirmation import PurchaseConfirmation
from koalixcrm.crm.documents.purchase_order import PurchaseOrder
from koalixcrm.crm.documents.quote import Quote
from koalixcrm.crm.documents.payment_reminder import PaymentReminder
from koalixcrm.crm.documents.delivery_note import DeliveryNote

from koalixcrm.crm.product.currency import *
from koalixcrm.crm.product.price import *
from koalixcrm.crm.product.product import *
from koalixcrm.crm.product.product_type import *
from koalixcrm.crm.product.product_price import *
from koalixcrm.crm.product.customer_group_transform import *
from koalixcrm.crm.product.unit_transform import *
from koalixcrm.crm.product.tax import *
from koalixcrm.crm.product.unit import *

from koalixcrm.crm.reporting.agreement import *
from koalixcrm.crm.reporting.agreement_status import *
from koalixcrm.crm.reporting.agreement_type import *
from koalixcrm.crm.reporting.agreement_status import *
from koalixcrm.crm.reporting.estimation import *
from koalixcrm.crm.reporting.estimation_status import *
from koalixcrm.crm.reporting.human_resource import *
from koalixcrm.crm.reporting.resource_manager import *
from koalixcrm.crm.reporting.resource_type import *
from koalixcrm.crm.reporting.resource_price import *
from koalixcrm.crm.reporting.generic_task_link import *
from koalixcrm.crm.reporting.task import *
from koalixcrm.crm.reporting.task_link_type import *
from koalixcrm.crm.reporting.task_status import *
from koalixcrm.crm.reporting.work import *
from koalixcrm.crm.reporting.project import *
from koalixcrm.crm.reporting.project_link_type import *
from koalixcrm.crm.reporting.project_status import *
from koalixcrm.crm.reporting.generic_project_link import *
from koalixcrm.crm.reporting.reporting_period import *
from koalixcrm.crm.reporting.reporting_period_status import *









# -*- coding: utf-8 -*-

from koalixcrm.crm.models import DeliveryNote
from koalixcrm.crm.factories.factory_sales_document import StandardSalesDocumentFactory


class StandardDeliveryNoteFactory(StandardSalesDocumentFactory):
    class Meta:
        model = DeliveryNote

    tracking_reference = "This is a tracking reference"
    status = "S"




# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-05 21:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('crm', '0012_auto_20180105_1840'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='purchaseconfirmation',
            name='status',
        ),
        migrations.RemoveField(
            model_name='purchaseconfirmation',
            name='valid_until',
        ),
        migrations.AddField(
            model_name='deliverynote',
            name='derived_from_quote',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Quote'),
        ),
        migrations.AddField(
            model_name='deliverynote',
            name='tracking_reference',
            field=models.CharField(blank=True, max_length=100, verbose_name='Tracking Reference'),
        ),
        migrations.AddField(
            model_name='paymentreminder',
            name='payable_until',
            field=models.DateField(default=django.utils.timezone.now, verbose_name='To pay until'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='paymentreminder',
            name='payment_bank_reference',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Payment Bank Reference'),
        ),
        migrations.AddField(
            model_name='purchaseconfirmation',
            name='derived_from_quote',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Quote'),
        ),
        migrations.AddField(
            model_name='purchaseorder',
            name='template_set',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.DocumentTemplate', verbose_name='Referred Template'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from koalixcrm.crm.const.status import *
from koalixcrm.crm.documents.sales_document import SalesDocument, OptionSalesDocument
from koalixcrm.plugin import *


class PurchaseOrder(SalesDocument):
    supplier = models.ForeignKey("Supplier", on_delete=models.CASCADE, verbose_name=_("Supplier"), null=True)
    status = models.CharField(max_length=1, choices=PURCHASEORDERSTATUS)

    def create_from_reference(self, calling_model):
        self.create_sales_document(calling_model)
        self.status = 'O'
        self.template_set = self.contract.get_template_set(self)
        self.save()
        self.attach_sales_document_positions(calling_model)
        self.attach_text_paragraphs()
        self.staff = calling_model.staff

    def __str__(self):
        return _("Purchase Order") + ": " + self.id.__str__() + " " + _("from Contract") + ": " + self.contract.id.__str__()

    class Meta:
        app_label = "crm"
        verbose_name = _('Purchase Order')
        verbose_name_plural = _('Purchase Orders')


class OptionPurchaseOrder(OptionSalesDocument):
    list_display = OptionSalesDocument.list_display + ('supplier', 'status',)
    list_filter = OptionSalesDocument.list_filter + ('status',)
    ordering = OptionSalesDocument.ordering
    search_fields = OptionSalesDocument.search_fields
    fieldsets = OptionSalesDocument.fieldsets + (
        (_('Purchase Order specific'), {
            'fields': ('supplier', 'status',)
        }),
    )

    save_as = OptionSalesDocument.save_as
    inlines = OptionSalesDocument.inlines
    actions = ['create_purchase_confirmation', 'create_invoice', 'create_quote',
               'create_delivery_note', 'create_pdf',
               'register_invoice_in_accounting', 'register_payment_in_accounting',]

    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("quoteInlines"))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-04-22 20:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('accounting', '0005_auto_20171110_1732'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['account_number'], 'verbose_name': 'Account', 'verbose_name_plural': 'Account'},
        ),
        migrations.RenameField(
            model_name='account',
            old_name='accountNumber',
            new_name='account_number',
        ),
        migrations.RenameField(
            model_name='account',
            old_name='accountType',
            new_name='account_type',
        ),
        migrations.RenameField(
            model_name='account',
            old_name='isACustomerPaymentAccount',
            new_name='is_a_customer_payment_account',
        ),
        migrations.RenameField(
            model_name='account',
            old_name='isopeninterestaccount',
            new_name='is_open_interest_account',
        ),
        migrations.RenameField(
            model_name='account',
            old_name='isopenreliabilitiesaccount',
            new_name='is_open_reliabilities_account',
        ),
        migrations.RenameField(
            model_name='account',
            old_name='isProductInventoryActiva',
            new_name='is_product_inventory_activa',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='accountingPeriod',
            new_name='accounting_period',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='bookingDate',
            new_name='booking_date',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='bookingReference',
            new_name='booking_reference',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='dateofcreation',
            new_name='date_of_creation',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='fromAccount',
            new_name='from_account',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='lastmodification',
            new_name='last_modification',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='lastmodifiedby',
            new_name='last_modified_by',
        ),
        migrations.RenameField(
            model_name='booking',
            old_name='toAccount',
            new_name='to_account',
        ),
        migrations.RenameField(
            model_name='productcategorie',
            old_name='lossAccount',
            new_name='loss_account',
        ),
        migrations.RenameField(
            model_name='productcategorie',
            old_name='profitAccount',
            new_name='profit_account',
        ),
        migrations.AddField(
            model_name='accountingperiod',
            name='template_set',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.DocumentTemplate', verbose_name='Referred Template'),
        ),
    ]



# -*- coding: utf-8 -*-

from decimal import Decimal
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _


class Currency(models.Model):
    description = models.CharField(verbose_name=_("Description"),
                                   max_length=100)
    short_name = models.CharField(verbose_name=_("Displayed Name After Prices"),
                                  max_length=3)
    rounding = models.DecimalField(max_digits=5,
                                   decimal_places=2,
                                   verbose_name=_("Rounding"),
                                   blank=True,
                                   null=True)

    def get_rounding(self):
        """Returns either the stored rounding value for a currency or a default rounding value of 0.05

        Args: no arguments

        Returns: Decimal value

        Raises: should not return exceptions"""
        if self.rounding is None:
            return Decimal('0.05')
        else:
            return self.rounding

    def round(self, value):
        """Rounds the input value to the rounding resolution which is defined in the variable "rounding"

        Args: Decimal value value

        Returns: Decimal value

        Raises: should not return exceptions"""
        rounded_value = int(value / self.get_rounding()) * self.get_rounding()
        return rounded_value

    def __str__(self):
        return self.short_name

    class Meta:
        app_label = "crm"
        verbose_name = _('Currency')
        verbose_name_plural = _('Currency')


class OptionCurrency(admin.ModelAdmin):
    list_display = ('id',
                    'description',
                    'short_name',
                    'rounding')
    fieldsets = (('', {'fields': ('description',
                                  'short_name',
                                  'rounding')}),)
    allow_add = True



# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import Estimation
from koalixcrm.crm.factories.factory_resource import StandardResourceFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_estimation_status import StartedEstimationStatusFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardEstimationToTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Estimation

    amount = "112.50"
    task = factory.SubFactory(StandardTaskFactory)
    resource = factory.SubFactory(StandardResourceFactory)
    date_from = make_date_utc(datetime.datetime(2018, 5, 2, 00)).date()
    date_until = make_date_utc(datetime.datetime(2018, 6, 15, 00)).date()
    status = factory.SubFactory(StartedEstimationStatusFactory)
    reporting_period = factory.SubFactory(StandardReportingPeriodFactory)


class StandardHumanResourceEstimationToTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Estimation

    amount = "112.50"
    task = factory.SubFactory(StandardTaskFactory)
    resource = factory.SubFactory(StandardHumanResourceFactory)
    date_from = make_date_utc(datetime.datetime(2018, 5, 2, 00)).date()
    date_until = make_date_utc(datetime.datetime(2018, 6, 15, 00)).date()
    status = factory.SubFactory(StartedEstimationStatusFactory)
    reporting_period = factory.SubFactory(StandardReportingPeriodFactory)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-20 18:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0053_auto_20181014_2305'),
    ]

    operations = [
        migrations.AlterField(
            model_name='currency',
            name='short_name',
            field=models.CharField(max_length=3, verbose_name='Displayed Name After Prices'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-05 17:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accountNumber', models.IntegerField(verbose_name='Account Number')),
                ('title', models.CharField(max_length=50, verbose_name='Account Title')),
                ('accountType', models.CharField(
                    choices=[('E', 'Earnings'), ('S', 'Spendings'), ('L', 'Liabilities'), ('A', 'Assets')],
                    max_length=1, verbose_name='Account Type')),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
                ('originalAmount',
                 models.DecimalField(decimal_places=2, default=0.0, max_digits=20, verbose_name='Original Amount')),
                ('isopenreliabilitiesaccount', models.BooleanField(verbose_name='Is The Open Liabilities Account')),
                ('isopeninterestaccount', models.BooleanField(verbose_name='Is The Open Interests Account')),
                ('isProductInventoryActiva', models.BooleanField(verbose_name='Is a Product Inventory Account')),
                ('isACustomerPaymentAccount', models.BooleanField(verbose_name='Is a Customer Payment Account')),
            ],
            options={
                'ordering': ['accountNumber'],
                'verbose_name': 'Account',
                'verbose_name_plural': 'Account',
            },
        ),
        migrations.CreateModel(
            name='AccountingPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=200, verbose_name='Title')),
                ('begin', models.DateField(verbose_name='Begin')),
                ('end', models.DateField(verbose_name='End')),
            ],
            options={
                'verbose_name': 'Accounting Period',
                'verbose_name_plural': 'Accounting Periods',
            },
        ),
        migrations.CreateModel(
            name='Booking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Amount')),
                ('description', models.CharField(blank=True, max_length=120, null=True, verbose_name='Description')),
                ('bookingDate', models.DateTimeField(verbose_name='Booking at')),
                ('dateofcreation', models.DateTimeField(auto_now=True, verbose_name='Created at')),
                ('lastmodification', models.DateTimeField(auto_now_add=True, verbose_name='Last modified')),
                ('accountingPeriod',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='accounting.AccountingPeriod',
                                   verbose_name='AccountingPeriod')),
            ],
            options={
                'verbose_name': 'Booking',
                'verbose_name_plural': 'Bookings',
            },
        ),
        migrations.CreateModel(
            name='ProductCategorie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='Product Categorie Title')),
                ('lossAccount',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_loss_account',
                                   to='accounting.Account', verbose_name='Loss Account')),
                ('profitAccount',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='db_profit_account',
                                   to='accounting.Account', verbose_name='Profit Account')),
            ],
            options={
                'verbose_name': 'Product Categorie',
                'verbose_name_plural': 'Product Categories',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-04 20:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
        ('crm', '0007_auto_20171210_2126'),
    ]

    operations = [
        migrations.RenameField(
            model_name='textparagraphinsalescontract',
            old_name='SalesContract',
            new_name='sales_contract',
        ),
        migrations.AddField(
            model_name='contract',
            name='default_template_set',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.TemplateSet', verbose_name='Default Template Set'),
        ),
    ]



# -*- coding: utf-8 -*-
from os import path
from wsgiref.util import FileWrapper
from subprocess import CalledProcessError
from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from koalixcrm.crm.exceptions import *
from koalixcrm.djangoUserExtension.exceptions import *
from django.contrib import messages


class PDFExportView:

    @staticmethod
    def export_pdf(calling_model_admin, request, source, redirect_to, template_to_use, *args, **kwargs):
        """This method exports PDFs provided by different Models in the crm application

            Args:
              calling_model_admin (ModelAdmin):  The calling ModelAdmin must be provided for error message response.
              request: The request User is to know where to save the error message
              source:  The model from which a PDF should be exported
              redirect_to (str): String that describes to where the method should redirect in case of an error
              template_to_use (Template Set): For some documents that need to be created there exists more
              than one template with this parameter the template set can be set during the export function


            Returns:
              HTTpResponse with a PDF when successful
              HTTpResponseRedirect when not successful

            Raises:
              raises Http404 exception if anything goes wrong"""
        try:
            pdf = source.create_pdf(template_to_use, request.user, *args, **kwargs)
            response = HttpResponse(FileWrapper(open(pdf, 'rb')), content_type='application/pdf')
            response['Content-Length'] = path.getsize(pdf)
        except (TemplateSetMissing,
                TemplateSetMissingInContract,
                UserExtensionMissing,
                CalledProcessError,
                UserExtensionEmailAddressMissing,
                UserExtensionPhoneAddressMissing,
                TemplateSetMissingForUserExtension) as e:
            if isinstance(e, UserExtensionMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("User Extension Missing"),
                                                 level=messages.ERROR)
            elif isinstance(e, UserExtensionEmailAddressMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("User Extension Email Missing"),
                                                 level=messages.ERROR)
            elif isinstance(e, UserExtensionPhoneAddressMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("User Extension Phone Missing"),
                                                 level=messages.ERROR)
            elif isinstance(e, TemplateSetMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("Template-set Missing"),
                                                 level=messages.ERROR)
            elif isinstance(e, TemplateSetMissingInContract):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("Template-set Missing"),
                                                 level=messages.ERROR)
            elif isinstance(e, TemplateFOPConfigFileMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("Fop Config File Missing in TemplateSet"),
                                                 level=messages.ERROR)
            elif isinstance(e, TemplateXSLTFileMissing):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("XSLT File Missing in TemplateSet"),
                                                 level=messages.ERROR)
            elif isinstance(e, TemplateSetMissingForUserExtension):
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, _("Work report template missing in the user extension"),
                                                 level=messages.ERROR)
            elif type(e) == CalledProcessError:
                response = HttpResponseRedirect(redirect_to)
                calling_model_admin.message_user(request, e.output)
            else:
                raise Http404
        return response



import pytest
from django.test import LiveServerTestCase
from selenium import webdriver
from koalixcrm.test_support_functions import *
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory


class TimeTrackingAddRow(LiveServerTestCase):

    def setUp(self):
        firefox_options = webdriver.firefox.options.Options()
        firefox_options.set_headless(headless=True)
        self.selenium = webdriver.Firefox(firefox_options=firefox_options)
        self.test_user = AdminUserFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.test_user_extension = StandardUserExtensionFactory.create(user=self.test_user)
        self.test_human_resource = StandardHumanResourceFactory.create(user=self.test_user_extension)

    def tearDown(self):
        self.selenium.quit()

    @pytest.mark.front_end_tests
    def test_add_new_row(self):
        selenium = self.selenium
        # login
        selenium.get('%s%s' % (self.live_server_url, '/koalixcrm/crm/reporting/time_tracking/'))
        # the browser will be redirected to the login page
        timeout = 5
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_username'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        username = selenium.find_element_by_xpath('//*[@id="id_username"]')
        password = selenium.find_element_by_xpath('//*[@id="id_password"]')
        submit_button = selenium.find_element_by_xpath('/html/body/div/article/div/div/form/div/ul/li/input')
        username.send_keys("admin")
        password.send_keys("admin")
        submit_button.send_keys(Keys.RETURN)
        # after the login, the browser is redirected to the target url /koalixcrm/crm/reporting/time_tracking
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_form-0-project'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        # find the form element
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-project"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-task"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-date"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-start_time"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-stop_time"]')
        assert_when_element_does_not_exist(self, '//*[@id="id_form-0-description"]')
        assert_when_element_does_not_exist(self, 'save')
        # check existence of a second form before pressing add more
        assert_when_element_exists(self, '//*[@id="id_form-1-project"]')
        add_more_button = selenium.find_element_by_xpath('//*[@id="add_more"]')
        add_more_button.send_keys(Keys.RETURN)
        # check existence of a second form after pressing add more
        assert_when_element_does_not_exist(self, '//*[@id="id_form-1-project"]')



from rest_framework import serializers
from koalixcrm.crm.reporting.agreement_status import AgreementStatus


class OptionAgreementStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title', read_only=True)
    description = serializers.CharField(source='description', read_only=True)
    isAgreed = serializers.BooleanField(source='is_agreed', read_only=True)

    class Meta:
        model = AgreementStatus
        fields = ('title',
                  'description',
                  'is_agreed')


class AgreementStatusJSONSerializer(serializers.HyperlinkedModelSerializer):
    title = serializers.CharField(source='title')
    description = serializers.CharField(source='description')
    isAgreed = serializers.BooleanField(source='is_agreed')

    class Meta:
        model = AgreementStatus
        fields = ('title',
                  'description',
                  'is_agreed')



# -*- coding: utf-8 -*-

import factory
from koalixcrm.djangoUserExtension.models import TemplateSet


class StandardTemplateSetFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TemplateSet

    title = "Just an empty Template Set"



# -*- coding: utf-8 -*-

import factory
from koalixcrm.djangoUserExtension.models import UserExtension
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.djangoUserExtension.factories.factory_template_set import StandardTemplateSetFactory


class StandardUserExtensionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = UserExtension
        django_get_or_create = ('user',)

    user = factory.SubFactory(StaffUserFactory)
    default_template_set = factory.SubFactory(StandardTemplateSetFactory)
    default_currency = factory.SubFactory(StandardCurrencyFactory)



# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.reporting.project import Project
from koalixcrm.crm.reporting.task_status import TaskStatus
from koalixcrm.crm.rest.project_rest import OptionProjectJSONSerializer
from koalixcrm.crm.rest.task_status_rest import OptionTaskStatusJSONSerializer


class OptionTaskJSONSerializer(serializers.HyperlinkedModelSerializer):
    project = OptionProjectJSONSerializer(allow_null=False, read_only=True)
    status = OptionTaskStatusJSONSerializer(allow_null=False, read_only=True)
    lastStatusChange = serializers.DateField(source='last_status_change', read_only=True)
    isReportingAllowed = serializers.SerializerMethodField()

    class Meta:
        model = Task
        fields = ('id',
                  'title',
                  'project',
                  'description',
                  'status',
                  'lastStatusChange',
                  'isReportingAllowed',)

    def get_isReportingAllowed(self, obj):
        if obj.is_reporting_allowed():
            return "True"
        else:
            return "False"


class TaskJSONSerializer(serializers.HyperlinkedModelSerializer):
    project = OptionProjectJSONSerializer(allow_null=False)
    status = OptionTaskStatusJSONSerializer(allow_null=False)
    lastStatusChange = serializers.DateField(source='last_status_change')
    isReportingAllowed = serializers.SerializerMethodField()

    class Meta:
        model = Task
        fields = ('id',
                  'title',
                  'project',
                  'description',
                  'status',
                  'lastStatusChange',
                  'isReportingAllowed',)

    def get_isReportingAllowed(self, obj):
        if obj.is_reporting_allowed():
            return "True"
        else:
            return "False"

    def create(self, validated_data):
        task = Task()
        # Deserialize project
        project = validated_data.pop('project')
        if project:
            if project.get('id', None):
                task.project = Project.objects.get(id=project.get('id', None))
            else:
                task.project = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', None):
                task.status = TaskStatus.objects.get(id=status.get('id', None))
            else:
                task.status = None
        task.title = validated_data['title']
        task.description = validated_data['description']
        task.last_status_change = validated_data['lastStatusChange']
        task.save()
        return task

    def update(self, task, validated_data):
        # Deserialize project
        project = validated_data.pop('project')
        if project:
            if project.get('id', task.project):
                task.project = Project.objects.get(id=project.get('id', None))
            else:
                task.project = task.project_id
        else:
            task.project = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', task.status):
                task.status = TaskStatus.objects.get(id=status.get('id', None))
            else:
                task.status = task.status_id
        else:
            task.status = None
        task.title = validated_data['title']
        task.description = validated_data['description']
        task.last_status_change = validated_data['lastStatusChange']
        task.save()
        return task






# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Contract
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.djangoUserExtension.factories.factory_template_set import StandardTemplateSetFactory


class StandardContractFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Contract

    staff = factory.SubFactory(StaffUserFactory)
    description = "This is the description to a test contract"
    default_customer = factory.SubFactory(StandardCustomerFactory)
    default_currency = factory.SubFactory(StandardCurrencyFactory)
    default_template_set = factory.SubFactory(StandardTemplateSetFactory)
    date_of_creation = "2018-05-01"
    last_modification = "2018-05-03"
    last_modified_by = factory.SubFactory(StaffUserFactory)



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Contact
from koalixcrm.crm.factories.factory_user import StaffUserFactory


class StandardContactFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Contact
        django_get_or_create = ('name',)

    name = "John Smith"
    date_of_creation = "2018-05-01"
    last_modification = "2018-05-03"
    last_modified_by = factory.SubFactory(StaffUserFactory)



# -*- coding: utf-8 -*-
import sys

from django.conf import settings


class PluginProcessor(object):
    @staticmethod
    def converttorelativestring(pluginmodule, nameofinline):
        output = []
        if len(nameofinline) != 0:
            output.append(pluginmodule.__name__ + "." + nameofinline[0])
            return output
        else:
            return []

    @staticmethod
    def getAllPlugins():
        allpluginmodules = []
        for plugin in settings.KOALIXCRM_PLUGINS:
            temp = __import__(plugin + ".admin")
            allpluginmodules.append(sys.modules[plugin + ".admin"]);
        return allpluginmodules

    @staticmethod
    def resolve_name(name, package, level):
        """Return the absolute name of the module to be imported."""
        if not hasattr(package, 'rindex'):
            raise ValueError("'package' not set to a string")
        dot = len(package)
        for x in xrange(level, 1, -1):
            try:
                dot = package.rindex('.', 0, dot)
            except ValueError:
                raise ValueError("attempted relative import beyond top-level "
                                 "package")
        return "%s.%s" % (package[:dot], name)

    @staticmethod
    def import_module(name, package=None):
        """Import a module.

        The 'package' argument is required when performing a relative import. It
        specifies the package to use as the anchor point from which to resolve the
        relative import to an absolute import.

        """
        if name.startswith('.'):
            if not package:
                raise TypeError("relative imports require the 'package' argument")
            level = 0
            for character in name:
                if character != '.':
                    break
                    level += 1
            name = resolve_name(name[level:], package, level)
        __import__(name)
        return sys.modules[name]

    def getPluginAdditions(self, additionname):
        listofAdditions = []
        allpluginmodules = self.getAllPlugins()
        for pluginmodule in allpluginmodules:
            try:
                listofAdditions.extend(getattr(pluginmodule.KoalixcrmPluginInterface, additionname))
            except AttributeError:
                continue
        return listofAdditions


# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-17 20:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0021_purchaseorder'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='purchaseorder',
            options={'verbose_name': 'Purchase Order', 'verbose_name_plural': 'Purchase Orders'},
        ),
        migrations.AlterField(
            model_name='purchaseorder',
            name='supplier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.Supplier', verbose_name='Supplier'),
        ),
    ]



# -*- coding: utf-8 -*-
import pytest
import os
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium import webdriver
from koalixcrm.test_support_functions import *
from koalixcrm.crm.factories.factory_contract import StandardContractFactory
from koalixcrm.crm.factories.factory_quote import StandardQuoteFactory
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.djangoUserExtension.factories.factory_document_template import StandardQuoteTemplateFactory
from koalixcrm.djangoUserExtension.factories.factory_document_template import StandardInvoiceTemplateFactory
from koalixcrm.djangoUserExtension.factories.factory_document_template import StandardPurchaseOrderTemplateFactory
from koalixcrm.crm.documents.quote import Quote
from koalixcrm.crm.documents.invoice import Invoice
from koalixcrm.crm.documents.purchase_order import PurchaseOrder


class CreateSalesDocumentFromContract(StaticLiveServerTestCase):

    @classmethod
    def setUpClass(cls):
        super(CreateSalesDocumentFromContract, cls).setUpClass()
        firefox_options = webdriver.firefox.options.Options()
        firefox_options.set_headless(headless=True)
        cls.selenium = webdriver.Firefox(firefox_options=firefox_options)
        cls.selenium.implicitly_wait(10)
        cls.test_user = AdminUserFactory.create()
        cls.test_customer_group = StandardCustomerGroupFactory.create()
        cls.test_contract = StandardContractFactory.create()
        cls.test_quote = StandardQuoteFactory.create(contract=cls.test_contract)
        cls.test_quote_template = StandardQuoteTemplateFactory.create()
        cls.test_invoice_template = StandardInvoiceTemplateFactory.create()
        cls.test_purchase_order_template = StandardPurchaseOrderTemplateFactory.create()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(CreateSalesDocumentFromContract, cls).tearDownClass()

    def tearDown(self):
        if len(self._outcome.errors) > 0:
            directory = os.getcwd() + "/test_results/Screenshots/"
            if not os.path.exists(directory):
                os.makedirs(directory)
            self.selenium.save_screenshot(directory + "%s.png" % "test_name")
        super(CreateSalesDocumentFromContract, self).tearDown()

    @pytest.mark.front_end_tests
    def test_create_sales_document_from_quote(self):
        selenium = self.selenium
        # login
        selenium.get('%s%s' % (self.live_server_url, '/admin/crm/quote/'))
        # the browser will be redirected to the login page
        timeout = 2
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_username'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")
        username = selenium.find_element_by_xpath('//*[@id="id_username"]')
        password = selenium.find_element_by_xpath('//*[@id="id_password"]')
        submit_button = selenium.find_element_by_xpath('/html/body/div/article/div/div/form/div/ul/li/input')
        username.send_keys("admin")
        password.send_keys("admin")
        submit_button.send_keys(Keys.RETURN)
        # after the login, the browser is redirected to the target url /koalixcrm/crm/contract
        try:
            element_present = expected_conditions.presence_of_element_located((By.ID, 'id_form-0-project'))
            WebDriverWait(selenium, timeout).until(element_present)
        except TimeoutException:
            print("Timed out waiting for page to load")

        test_parameters = {Quote: {"action_name": "create_quote",
                                   "template_name": "quote_template",
                                   "template_to_select": self.test_quote_template},
                           Invoice: {"action_name": "create_invoice",
                                     "template_name": "invoice_template",
                                     "template_to_select": self.test_invoice_template},
                           PurchaseOrder: {"action_name": "create_purchase_order",
                                           "template_name": "purchase_order_template",
                                           "template_to_select": self.test_purchase_order_template},
                           }
        for document_type in test_parameters:
            test_parameter = test_parameters[document_type]
            create_sales_document_from_reference(test_case=self,
                                                 timeout=timeout,
                                                 document_type=document_type,
                                                 reference_type="quote",
                                                 reference_id=self.test_quote,
                                                 action_name=test_parameter["action_name"],
                                                 template_name=test_parameter["template_name"],
                                                 template_to_select=test_parameter["template_to_select"])



# -*- coding: utf-8 -*

from django.utils.translation import ugettext as _

INVOICESTATUS = (
    ('P', _('Payed')),
    ('C', _('Invoice created')),
    ('I', _('Invoice sent')),
    ('F', _('First reminder sent')),
    ('R', _('Second reminder sent')),
    ('U', _('Customer cant pay')),
    ('D', _('Deleted')),
)

QUOTESTATUS = (
    ('S', _('Success')),
    ('I', _('Quote created')),
    ('Q', _('Quote sent')),
    ('F', _('First reminder sent')),
    ('R', _('Second reminder sent')),
    ('D', _('Deleted')),
)

PURCHASEORDERSTATUS = (
    ('O', _('Ordered')),
    ('D', _('Delayed')),
    ('Y', _('Delivered')),
    ('I', _('Invoice registered')),
    ('P', _('Invoice payed')),
)

DELIVERYNOTESTATUS = (
    ('C', _('Created')),
    ('S', _('Sent')),
    ('R', _('Received')),
    ('R', _('Lost')),
)

CALLSTATUS = (
    ('P', _('Planned')),
    ('D', _('Delayed')),
    ('R', _('ToRecall')),
    ('F', _('Failed')),
    ('S', _('Success')),
)
